Rem  Copyright (c) Oracle Corporation 1999 - 2017. All Rights Reserved.
Rem
Rem    NAME
Rem      apxpatch_cdb.sql
Rem
Rem    DESCRIPTION
Rem      This script is called by apxpatch.sql and *SHOULD NOT BE RUN* directly.
Rem
Rem    NOTES
Rem      Assumes the SYS user is connected. Must be run locally to the database and the
Rem      ORACLE_HOME environment variable must be set.
Rem
Rem    REQUIREMENTS
Rem      - Oracle Database 12.1.0.1 or later
Rem
Rem    MODIFIED   (MM/DD/YYYY)
Rem     vuvarov    12/22/2016 - Created
Rem     vuvarov    03/29/2017 - Updated for APEX 5.1.2
Rem    hfarrell    06/26/2017 - Updated for APEX 5.1.3
Rem    hfarrell    09/25/2017 - Updated for APEX 5.1.4


set define '^' verify off
define APPUN        = 'APEX_050100'
define PREFIX       = '@'
define INSTALL_TYPE = 'MANUAL'

@^PREFIX.patches/5_1_4/prereq.sql

timing start "Complete Patch"


whenever sqlerror exit

column :xe_home new_value OH_HOME NOPRINT
variable xe_home varchar2(255)

set serverout on
begin
-- get oracle_home
    sys.dbms_system.get_env('ORACLE_HOME',:xe_home);
    if length(:xe_home) = 0 then
        sys.dbms_output.put_line(lpad('-',80,'-'));
        raise_application_error (
            -20001,
            'Oracle Home environment variable not set' );
    end if;
end;
/
whenever sqlerror continue

set termout off
select :xe_home from sys.dual;
set termout on

prompt Performing installation in multitenant container database in the background.
prompt The installation progress is spooled into *patch_con*.log files.
prompt
prompt Please wait...
prompt

host ^OH_HOME/perl/bin/perl -I ^OH_HOME/rdbms/admin ^OH_HOME/rdbms/admin/catcon.pl -b syspatch_con patches/5_1_4/syspatch.sql --p^PREFIX --p^INSTALL_TYPE

host ^OH_HOME/perl/bin/perl -I ^OH_HOME/rdbms/admin ^OH_HOME/rdbms/admin/catcon.pl -b ddlpatch_con patches/5_1_4/ddlpatch.sql --p^PREFIX --p^INSTALL_TYPE

begin
    sys.dbms_utility.compile_schema('^APPUN.', false);
end;
/

host ^OH_HOME/perl/bin/perl -I ^OH_HOME/rdbms/admin ^OH_HOME/rdbms/admin/catcon.pl -b deppatch_con patches/5_1_4/deppatch.sql --p^PREFIX --p^INSTALL_TYPE

begin
    sys.dbms_utility.compile_schema('^APPUN.', false);
end;
/

host ^OH_HOME/perl/bin/perl -I ^OH_HOME/rdbms/admin ^OH_HOME/rdbms/admin/catcon.pl -b corepatch_con patches/5_1_4/corepatch.sql --p^PREFIX --p^INSTALL_TYPE

timing stop

prompt
prompt Installation completed. Log files for each container can be found in:
prompt
prompt syspatch_con*.log
prompt ddlpatch_con*.log
prompt deppatch_con*.log
prompt corepatch_con*.log
prompt
prompt You can quickly scan for ORA errors or compilation errors by using a utility
prompt like grep:
prompt
prompt grep ORA- *.log
prompt grep PLS- *.log
prompt
