/*global apex,alert,window,$v*/
/**
 @license
 Oracle Database Application Express, Release 5.0
 Copyright (c) 2013, 2017, Oracle. All rights reserved.
 */
/**
 * @fileOverview
 * Developer Toolbar controller
 *
 * depends on:
 *   util
 *   navigation
 *   storage
 *   widget.menu
 * Optional integration with ui.dialog
 **/
(function( $, util, nav, storage ) {
    "use strict";

    var BUILDER_WINDOW_NAME = "APEX_BUILDER", // keep in sync with builder.js
        DEV_TOOLBAR_KEY = "ORA_WWV_apex.builder.devToolbar",
        THEMEROLLER_KEY = "ORA_WWV_apex.builder.themeRoller", // keep in sync with utr.js
        QUICK_EDIT_CURSOR = "crosshair",
        IMAGES_DIRECTORY = window.apex_img_dir,
        THEMEROLLER_BASE = IMAGES_DIRECTORY + "apex_ui/theme_roller/",
        CODEMIRROR_BASE = IMAGES_DIRECTORY + "libraries/codemirror/5.16/",
        COLORPICKER_BASE = IMAGES_DIRECTORY + "libraries/d3.oracle.colorpicker/0.1/",
        PALETTE_BASE = IMAGES_DIRECTORY + "libraries/d3.oracle.palette/0.1/",
        D3_BASE = IMAGES_DIRECTORY + "libraries/d3/3.3.11/";

    var gDevToolbarSessionStore = storage.getScopedSessionStorage( { prefix: DEV_TOOLBAR_KEY, useAppId: false } ),
        gDevToolbarLocalStore =  storage.getScopedLocalStorage( { prefix: DEV_TOOLBAR_KEY, useAppId: false } ),
        gThemeRollerSessionStore =  storage.getScopedSessionStorage( { prefix: THEMEROLLER_KEY, useAppId: true } );

    /*
     * If jQuery dialog is available make it dev toolbar aware
     * This allows the dev toolbar to be used while a dialog is open
     */
    if ( $.ui.dialog ) {
        $.widget( "ui.dialog", $.ui.dialog, {
            _allowInteraction: function(event) {
                // todo is overlay class needed?
                return $( event.target ).closest(".a-DevToolbar, #apexDevToolbarMenu, .u-Overlay--quickEdit").length > 0 || this._super(event);
            }
        });
    }

    function getWindowWidth() {
        return document.documentElement.clientWidth;
    }

    //todo proposed facility for taking and displaying time measurements
    var perfMap = [
        ["Redirect", "redirectStart", "redirectEnd", "blue"],
        ["Unload", "unloadEventStart", "unloadEventEnd", "blue"],
        ["Cache", "fetchStart", "domainLookupStart", "lightgray"],
        ["DNS", "domainLookupStart", "domainLookupEnd", "lightblue"],
        ["Connect", "connectStart", "connectEnd", "orange"],
        ["Request", "requestStart", "responseStart", "green"],
        ["Response", "responseStart", "responseEnd", "blue"],
        ["Processing", "responseEnd", "loadEventStart", "#8A2BE2"],
        ["DOM Loading", "domLoading", "domInteractive", "#9966CC"],
        ["DOM Interactive", "domInteractive", "domComplete", "#9370DB"],
        ["DOM Content Loaded", "domContentLoadedEventStart", "domContentLoadedEventEnd"],
        ["On Load", "loadEventStart", "loadEventEnd"]
    ];
    var navType = {
        "0": "Page Navigate",
        "1": "Page Reload",
        "2": "Page Forward or Back"
    };

    if ( window.performance && window.performance.timing && window.performance.navigation ) {
        apex.displayPageTiming = function( theWindow ) {
            var timing, rule$, time$, data$, scaleFactor,
                perfDlg$ =$( "<div id='apex-perf-dlg' style='padding: 4px;'><div class='perf'></div><div class='perfRule'></div>" +
                    "<div class='perfTime'></div><textarea class='perfData' style='position:absolute;top:0;left:-9999px;'></textarea></div>" ),
                width = getWindowWidth() - 20;

            // todo make general and move to apex.locale. requires recent browser
            function format(n) {
                return n.toLocaleString(apex.locale.getLanguage(), {useGrouping: true, maximumFractionDigits:3});
            }

            function focusTextArea() {
                data$.focus();
                data$[0].select();
            }

            theWindow = theWindow || window;
            timing = theWindow.performance.timing;

            // todo consider zoom or set start/end of display range. Consider save to session storage/export to compare across runs
            $("body").append(perfDlg$);
            rule$ = perfDlg$.find(".perfRule");
            time$ = perfDlg$.find(".perfTime");
            data$ = perfDlg$.find(".perfData");
            perfDlg$.dialog({
                title: "Page Load Timing",
                dialogClass: "apex-perf",
                modal: true,
                height: 300,
                width: width,
                resizable: false,
                open: function() {
                    var i, el$, begin, box, p, event, label, e, entries,
                        text = "",
                        events = [],
                        rangeBegin = timing.navigationStart,
                        rangeEnd = Math.max( timing.loadEventEnd, timing.domContentLoadedEventEnd, timing.loadEventEnd, timing.responseEnd ),
                        range = rangeEnd - rangeBegin,
                        w = perfDlg$.innerWidth() - 200, // leave room for labels xxx 200
                        c$ = perfDlg$.children().first();

                    events.push({ begin: rangeBegin, end: rangeEnd, duration: range, name: navType[theWindow.performance.navigation.type], color: "lightgray" } );
                    for ( i = 0; i < perfMap.length; i++ ) {
                        p = perfMap[i];
                        event = {
                            name: p[0],
                            begin: timing[p[1]]
                        };
                        if ( p[2] ) {
                            event.end = timing[p[2]];
                            event.duration = event.end - event.begin;
                        }
                        if ( event.end === 0 || event.begin === 0 || event.duration === 0 ) {
                            continue;
                        }
                        if ( p[3] ) {
                            event.color = p[3];
                        }
                        events.push( event );
                    }
                    // todo marks?
                    if ( performance.getEntriesByType ) {
                        entries = performance.getEntriesByType("measure");
                        for ( i = 0; i < entries.length; i++ ) {
                            e = entries[i];
                            event = {
                                name: e.name,
                                begin: timing.navigationStart + e.startTime,
                                duration: e.duration
                            };
                            event.end = event.begin + e.duration;
                            if ( event.end > rangeEnd ) {
                                rangeEnd = event.end;
                            }
                            events.push( event );
                        }
                    }
                    events.sort(function(a,b) {
                        return a.begin - b.begin;
                    });
                    scaleFactor = w / ( rangeEnd - rangeBegin );

                    text += "Event name\tBegin\tEnd\tDuration\r\n";
                    for ( i = 0; i < events.length; i++ ) {
                        box = events[i];
                        label = box.name;
                        if ( box.duration ) {
                            label += " " + format( box.duration ) + "ms"
                        }
                        el$ = $("<div class='perfEntry'><span class='perfBox' title='" + label + "'></span><span class='perfLabel'>" + label + "</span></div>");
                        begin = scaleFactor * ( box.begin - rangeBegin );
                        el$.children().first().css( {
                            backgroundColor: box.color || "red",
                            marginLeft: begin,
                            width: box.duration ? Math.max( 2, scaleFactor * box.duration ) : 2
                        });
                        c$.append(el$);
                        text += box.name + "\t" + ( box.begin - rangeBegin ) + "\t" + ( box.end ? box.end - rangeBegin : 0 ) + "\t" + ( box.duration || 0 ) + "\r\n";
                    }
                    rule$.height( Math.max( c$.height(), perfDlg$.height()) ); // set rule height
                    data$.val( text );
                    focusTextArea();
                },
                close: function() {
                    perfDlg$.remove();
                }
            })
            .on("mousemove", function(event) {
                var w,
                    offset = perfDlg$.offset(),
                    l = event.pageX - offset.left + 10 + perfDlg$[0].scrollLeft,
                    t = event.pageY - offset.top - 28 + perfDlg$[0].scrollTop;

                if ( t < 0 ) {
                    t = 0;
                }
                // xxx get info from event.target
                // xxx need to consider scroll offset do more to keep the time off the line and away from right edge
                rule$.css("left", l );
                time$.text( format( l / scaleFactor ) + "ms" );
                w = time$.width();
                if ( l + w + 40 > perfDlg$.width() ) {
                    l -= w + 10;
                }
                time$.css( {
                    left: l,
                    top: t
                } );
            })
            .on("mouseenter", function() {
                rule$.show();
                time$.show();
            })
            .on("mouseleave", function() {
                rule$.hide();
                time$.hide();
            })
            .on("click", function() {
                focusTextArea();
            })
            .on("focusin", function(event) {
                if ( event.target.nodeName !== "TEXTAREA" ) {
                    focusTextArea();
                }
            });
        };

        // todo move this to apex or apex.debug namespace even if empty without dev toolbar
        apex.timeCall = function( label, fn, context ) {
            var f = function( ) {
                var ret,
                    bl = label + "_begin",
                    el = label + "_end";

                // todo consider a polyfill for mark/measure
                if ( performance.mark ) {
                    performance.mark( bl );
                }
                ret = fn.apply( context || this, arguments );
                if ( performance.mark ) {
                    performance.mark( el );
                    performance.measure( label, bl, el );
                }
                return ret;
            };
            // f.length = fn.length; xxx was getting readonly error some times
            return f;
        };
    }

    //
    // Begin private dev toolbar code
    //
    var keys = $.ui.keyCode;
    var gContextStack = [],
        // For quick edit component locator
        boxes = null,
        overlay = null,
        outline = null,
        liveTemplateOptionsButton$ = null,
        started = false,
        storedStylesheet = null,
        storedCursor = null,
        scrollDelta = 40; // when something is scrolled into view this is the extra amount to scroll it

    function getThemeRollerInit(themeId, builderSessionId, callback) {
        var maxc = 5;
        var c = 0;
        var f = (function() {
            if ($.universalThemeRoller) {
                $.universalThemeRoller({
                    filePaths : {
                        utrScript: THEMEROLLER_BASE + "utr.js",
                        utrStylesheet: THEMEROLLER_BASE + "utr.css",
                        lessCompilerScript: THEMEROLLER_BASE + "less.js",
                        jQueryUiComponentsScript: THEMEROLLER_BASE + "jquery-ui.utr.js",
                        //colorPickerStylesheet: COLORPICKER_BASE + "css/colorpicker.css",
                        //colorPickerScript: COLORPICKER_BASE + "js/colorpicker.js",
                        codeMirrorScript: CODEMIRROR_BASE + "lib/codemirror.js",
                        codeMirrorCSSModeScript: CODEMIRROR_BASE + "mode/css/css.js",
                        codeMirrorStylesheet: CODEMIRROR_BASE + "lib/codemirror.css",
                        codeMirrorThemeStylesheet: CODEMIRROR_BASE + "theme/mbo.css",
                        d3Script: D3_BASE + "d3.min.js",
                        d3ColorPickerScript: COLORPICKER_BASE + "d3.oracle.colorpicker.js",
                        d3ColorPickerStylesheet: COLORPICKER_BASE + "d3.oracle.colorpicker.css",
                        d3PaletteScript: PALETTE_BASE + "d3.oracle.palette.js",
                        d3PaletteStylesheet: PALETTE_BASE + "d3.oracle.palette.css"
                    },
                    config : {
                        themeId: themeId,
                        builderSessionId: builderSessionId
                    }
                });
                if (callback && (typeof callback === 'function') ) {
                    callback();
                }
            } else if (c < maxc) {
                setTimeout( f, 100 );
                c++;
            }
        });
        return f;
    }

    function getUrl( button$ ) {
        return button$.attr("data-link");
    }

    // Return the opener window that is the apex builder window.
    function getApexBuilderFromOpenerChain( wnd ) {
        // if *this* is the builder window then don't care what the opener is
        // a builder opening the builder can result in a stale instance without this check
        if ( isBuilderWindow( wnd ) ) {
            return null;
        }
        try {
            if ( wnd.opener && !wnd.opener.closed && wnd.opener.apex && wnd.opener.apex.jQuery ) {
                // builder urls are in the 4000s
                if ( wnd.opener.location.href.match(/f?p=4\d\d\d:/) ||
                        wnd.opener.document.getElementById( "pFlowId" ).value.match(/^4\d\d\d$/) ) {
                    return wnd.opener;
                } else {
                    // Follow the opener chain to support non-modal (popup window) apex pages
                    return getApexBuilderFromOpenerChain( wnd.opener );
                }
            }
        } catch ( ex ) {
            return null; // window must contain a page from another domain
        }
        return null;
    }

    function isBuilderWindow(wnd) {
        return wnd.name && wnd.name.match( "^" + BUILDER_WINDOW_NAME );
    }

    function getBuilderInstance() {
        var builderWindow = getApexBuilderFromOpenerChain( window );
        if ( builderWindow ) {
            return builderWindow.document.getElementById( "pInstance" ).value;
        }
        return null;
    }

    function getBuilderUrl( url ) {
        var instance, parts;

        instance = getBuilderInstance();
        if ( instance ) {
            parts = url.split(":");
            parts[2] = instance;
            url = parts.join(":");
        }
        return url;
    }

    var contextPrototype = {
        navigateInPageDesigner: function( appId, pageId, typeId, componentId, errorFn ) {
            var builderWindow = getApexBuilderFromOpenerChain( window );

            if ( builderWindow && builderWindow.pageDesigner ) {
                builderWindow.pageDesigner.setPageSelection( appId, pageId, typeId, componentId, function( result ) {
                    if ( result !== "OK" && result !== "PAGE_CHANGE_ABORTED" ) {
                        errorFn();
                    }
                });
                // Focus the builder window now while still handling the click event even though controlling the page designer may still fail
                nav.openInNewWindow( "", BUILDER_WINDOW_NAME, { altSuffix: getBuilderInstance() } );
            } else {
                errorFn();
            }
        },
        builderWindow: function( action ) {
            this.builderWindowUrl( getBuilderUrl( this.actions[action] ) );
        },

        builderWindowUrl: function( url ) {
            var instance = getBuilderInstance();

            // if this is the builder window then don't try to manage another window just navigate
            if ( isBuilderWindow( window ) || this.windowMgmtMode === "NONE" ) {
                nav.redirect( url );
            } else {
                if ( !instance ) {
                    apex.message.confirm( this.text.noBuilderMessage, function(ok) {
                        if ( ok ) {
                            // just open the builder url in this window; turning this widow into a/the builder window
                            window.name = ""; // let the builder take over this window
                            nav.redirect( url );
                        }
                    } );
                } else {
                    nav.openInNewWindow( url, BUILDER_WINDOW_NAME, { altSuffix: instance } );
                }
            }
        },

        popup: function( action ) {
            nav.popup( {
                url:    getBuilderUrl( this.actions[action] ),
                name:   "view_debug",
                width:  1024,
                height: 768
            });
        },

        sameWindow: function( action ) {
            var match,
                url = this.actions[action];

            if (this.window === window) {
                nav.redirect( url );
            } else {
                // navigate in the nested iframe
                // we don't want to open or close and open the dialog again we just want to refresh the page
                match = /apex.navigation.dialog\(['"]([^'"]*)['"]/.exec(url);
                if ( match ) {
                    url = match[1];
                    // decode escaped characters
                    url = url.replace(/\\u(\d\d\d\d)/g, function(val, ch) {
                        return String.fromCharCode(parseInt(ch, 16));
                    });
                }
                this.window.location.href = url;
            }
        },

        // DOM locate code
        initLocateBoxes: function() {
            var i, comp$, pos, comp, fixed, op$, opPos,
                fixedBoxes = [];

            boxes = [];
            // go in reverse order so inner boxes are tested first
            for ( i = this.components.length - 1; i >= 0; i-- ) {
                comp = this.components[i];
                comp$ = $( "#" + util.escapeCSS( comp.domId ), this.document );

                if (comp.typeId === "5120") {
                    // for items it could be that the component DOM element is hidden; for example a hidden input
                    // so grab the closest visible element
                    comp$ = comp$.closest( ":visible" );
                } else {
                    // for all other component types exclude what is not visible
                    comp$ = comp$.filter( ":visible" );
                }
                // the check for body covers the case where items are in inline dialogs
                if ( !comp$.length || comp$[0] === document.body ) {
                    continue;
                }
                op$ = comp$.offsetParent();
                if ( op$.css("position") === "fixed" ) {
                    opPos = op$.position();
                    pos = comp$.position();
                    pos.top += opPos.top;
                    pos.left += opPos.left;
                    fixed = true;
                } else {
                    pos = comp$.offset();
                    fixed = false;
                }

                (fixed ? fixedBoxes : boxes).push({
                    node: comp$[0],
                    pageId: comp.pageId,
                    typeId: comp.typeId,
                    componentId: comp.id,
                    fixed: fixed,
                    top: pos.top,
                    bottom: pos.top + comp$.outerHeight(),
                    left: pos.left,
                    right: pos.left + comp$.outerWidth()
                });
            }
            // assume fixed things are on top so put at the start
            if ( fixedBoxes.length ) {
                boxes = fixedBoxes.concat( boxes );
            }
        },

        endDomLocate: function() {
            $( this.document ).off( ".locate" );
            $( document ).off( ".locate" );
            $( window ).off( ".locate" );
            $( this.document.body ).css( "cursor", storedCursor );
            storedStylesheet.remove();
            overlay.remove();
            outline.remove();
            liveTemplateOptionsButton$.remove();
            started = false;
            $( "#apexDevToolbarQuickEdit" ).focus();
        },

        beginDomLocate: function( action ) {
            var self = this,
                lastBox = null,
                lastBoxIndex = -1,
                body$ = $( this.document.body );

            var inBox = function( x, y, cx, cy, box ) {
                if ( box.fixed ) {
                    x = cx;
                    y = cy;
                }
                return ( y > box.top && y < box.bottom && x > box.left && x < box.right );
            };

            // Checks if a component supports Live Template Options by its typeID.
            var hasLiveTempOpt = function ( id ) {
                var NO_SUPPORT = {
                        "7040": true,   // IR column header
                        "7320": true,   // Classic report column
                        "7420": true,   // Tabular form column
                        "7940": true    // IG column
                };
                return !NO_SUPPORT[id];
            };

            var selectBox = function( scrollIfNeeded ) {
                var pos, h, w, node$, sp$, vpTop, vpBottom;

                if ( lastBox === null ) {
                    outline.hide();
                    liveTemplateOptionsButton$.hide();
                } else {
                    outline.show();
                    node$ = $( lastBox.node );
                    pos = node$.offset();
                    h = node$.outerHeight();
                    w = node$.outerWidth();

                    outline.css({top: pos.top + "px", left: pos.left + "px"});
                    // LiveTemplateOptionsButton is in an anonymous function primarily just to indicate that
                    // it depends on the variables outside of this closure but doesn't affect anything
                    // outside of the block.
                    if ( hasLiveTempOpt(lastBox.typeId) ) {
                        (function() {
                            liveTemplateOptionsButton$.show();
                            var left = pos.left + node$.outerWidth() - liveTemplateOptionsButton$.outerWidth() + 2;
                            var windowWidth = getWindowWidth();
                            var buttonWidth = liveTemplateOptionsButton$.outerWidth();
                            // Put it in the upper right corner of the outline box, but if it's too close to the  edge
                            // pad it to the left TODO: Consider just using jQuery UI's position lib?
                            if (left + buttonWidth > windowWidth) {
                                left = windowWidth - buttonWidth - 10;
                            }
                            liveTemplateOptionsButton$.css(
                                {
                                    top: pos.top + "px",
                                    left: left + "px"
                                }
                            );
                        })();
                    } else {
                        liveTemplateOptionsButton$.hide();
                    }
                    outline.height(h)
                        .width(w)
                        .focus();
                    if ( scrollIfNeeded ) {
                        sp$ = node$.scrollParent();
                        vpTop = apex.theme.defaultStickyTop();
                        vpBottom = ( sp$[0] === document ) ? $( window ).height() : sp$.height();
                        if ( pos.top < sp$.scrollTop() + vpTop ) {
                            sp$.scrollTop( pos.top - vpTop - scrollDelta );
                        } else if ( pos.top > sp$.scrollTop() + vpBottom ) {
                            sp$.scrollTop( pos.top + vpBottom + scrollDelta );
                        }
                    }
                }
            };

            this.initLocateBoxes();
            lastBox = null;
            // if locating in the top level document (the same one the toolbar is in) then the delegated click handler
            // added to the document will be hit ending the locating before we even begin so in that case delay start until after the click
            started = this.document !== document;
            overlay = $( "<div class='u-Overlay u-Overlay--quickEdit'></div>" ).appendTo( body$ );
            outline = $( "<div class='a-DevToolbar-uiSelector'  tabindex='-1'></div>" ).appendTo( body$ );
            liveTemplateOptionsButton$ = $(
                "<button class='a-DevToolbar-liveTemplateOptions'> " +
                    "<span class='a-Icon icon-util'></span>" +
                "</button>"
            ).appendTo( body$ );
            storedCursor = body$.css( "cursor" );
            storedStylesheet = $( "<style> *{ cursor: " + QUICK_EDIT_CURSOR + " !important; }</style>" ).appendTo( body$ );
            body$.css( "cursor", QUICK_EDIT_CURSOR );
            liveTemplateOptionsButton$.on("click", function() {
                if ( openBuilderIfNeeded() ) {
                    self.endDomLocate();
                    return false;
                }
                self.getTemplateOptions( lastBox );
                self.endDomLocate();
                return false;
            });

            $( this.document ).on( "mousemove.locate", function( event ) {
                var i, box, foundBox,
                    cx = event.clientX,
                    cy = event.clientY,
                    x = event.pageX,
                    y = event.pageY;

                if ( !started ) {
                    started = true;
                }
                foundBox = null;
                for ( i = 0; i < boxes.length; i++ ) {
                    box = boxes[i];
                    if ( inBox( x, y, cx, cy, box ) ) {
                        foundBox = box;
                        lastBoxIndex = i;
                        break;
                    }
                }
                if ( lastBox !== foundBox ) {
                    lastBox = foundBox;
                    selectBox();
                }
            } ).on("click.locate", function( event ) {
                if ( started ) {
                    self.endDomLocate();
                    if ( lastBox ) {
                        action( lastBox );
                    }
                    return false;
                } else {
                    started = true;
                }
            } );
            $( document ).on("keydown.locate", function( event ) {
                var kc = event.which;
                if ( kc === keys.ESCAPE ) {
                    self.endDomLocate();
                } else if ( kc === keys.DOWN ) {
                    if ( lastBoxIndex <= 0 ) {
                        lastBoxIndex = boxes.length - 1;
                    } else {
                        lastBoxIndex -= 1;
                    }
                    lastBox = boxes[lastBoxIndex];
                    selectBox( true );
                    event.preventDefault();
                } else if ( kc === keys.UP ) {
                    if ( lastBoxIndex < 0 || lastBoxIndex >= boxes.length - 1 ) {
                        lastBoxIndex = 0;
                    } else {
                        lastBoxIndex += 1;
                    }
                    lastBox = boxes[lastBoxIndex];
                    selectBox( true );
                    event.preventDefault();
                } else if ( kc === keys.TAB ) {
                    if ( lastBoxIndex > 0 ) {
                        if ( document.activeElement === outline[0] && liveTemplateOptionsButton$.is( ":visible" ))  {
                            liveTemplateOptionsButton$.focus();
                        } else {
                            outline.focus();
                        }
                        event.preventDefault();
                    }
                } else if ( ( kc === keys.SPACE || kc === keys.ENTER ) && document.activeElement === outline[0] ) {
                    self.endDomLocate();
                    if ( lastBox ) {
                        action( lastBox );
                    }
                    event.preventDefault();
                }
            } );
        },
        getTemplateOptions: function( el ) {
            var typeId = el.typeId;
            var componentId = el.componentId;
            var lSpinner$;
            if (!this.deferredTemplateOptionsInit) {
                this.deferredTemplateOptionsInit = [];
            }
            var deferred = this.deferredTemplateOptionsInit;
            // This will take a while. First, we need to check and then get, if needed, liveTemplateOptions widget
            // while concurrently fetching the template Data (two async operations running concurrently)
            // If and once liveTemplateOptions is loaded, then it itself needs to fetch its dependencies
            // Fortunately liveTemplateOptions takes care of its own dependencies without needi
            apex.util.delayLinger.start( "templateOptions", function() {
                lSpinner$ = util.showSpinner( $( el.node ) );
            });
            var initLiveTemplateOptions = function ( pData ) {
                if (!jQuery().liveTemplateOptions) {
                    // Live template options not loaded yet? no problem!
                    // we know that before this operation started we did a fetch for the livetemplateoptions
                    // so we'll just push this pData onto the deferred stack. once that fetch is done
                    // it will look at what's on this stack and then initialize the liveTemplateOptions
                    // that were waiting!
                    deferred.push(pData);
                    return;
                }
                apex.util.delayLinger.finish( "templateOptions", function() {
                    lSpinner$.remove();
                });
                $(el.node).liveTemplateOptions(
                    {
                        data: JSON.parse(JSON.stringify(pData)),
                        componentId: componentId,
                        onSave: function( submittedTemplateOptions, onSuccess, onFailure ) {
                            if ( openBuilderIfNeeded() ) {
                                return false;
                            }
                            apex.util.delayLinger.start( "templateOptions", function() {
                                lSpinner$ = util.showSpinner( $( el.node ) );
                            });
                            apex.server.process( "developerToolbar", {
                                p_flow_id:      4000,
                                p_flow_step_id: 0,
                                p_instance:     getBuilderInstance(),
                                x01:            "setTemplateOptions",
                                x02:            $v( "pFlowId" ),
                                x03:            $v( "pFlowStepId" ),
                                x04:            typeId,
                                x05:            componentId,
                                f01:            submittedTemplateOptions
                            }, {
                                success: function() {
                                    apex.util.delayLinger.finish( "templateOptions", function() {
                                        lSpinner$.remove();
                                    });
                                    onSuccess();
                                },
                                error: function( pData, err) {
                                    apex.util.delayLinger.finish("templateOptions", function () {
                                        lSpinner$.remove();
                                    });
                                    onFailure(pData, err);
                                }
                            });
                        }
                    }
                );
            };
            //Is live template options present?
            if (!jQuery().liveTemplateOptions) {
                // No? then we better get it.
                $.getScript( IMAGES_DIRECTORY + "apex_ui/js/" +
                    ( apex.debug.getLevel() === 0 ? "minified/widget.liveTemplateOptions.min.js" : "widget.liveTemplateOptions.js" ),
                    function( data, textStatus, jqxhr ) {
                        //Once we get it, then we need to preload its dependencies on the page.
                        $.apex.liveTemplateOptions.preload();
                        // Be sure to check if there were any templates that were waiting
                        // to be initialized with liveTemplateOptions
                        while (deferred.length > 0) {
                            initLiveTemplateOptions(deferred.pop());
                        }
                    });
            } else {
                // If it is present, then liveTemplateOptions can just go ahead with the preload
                // Note that this operation is idempotent after the first time, so we can call it as much as we want
                // with no reprecussions.
                $.apex.liveTemplateOptions.preload();
            }
            //TODO: cache the pData so the server doesn't get hit all the time?
            apex.server.process( "developerToolbar", {
                p_flow_id:      4000,
                p_flow_step_id: 0,
                p_instance:     this.builderSessionId,
                x01:            "getTemplateOptions",
                x02:            $v( "pFlowId" ),
                x03:            $v( "pFlowStepId" ),
                x04:            typeId,
                x05:            componentId
            }, {
                success: initLiveTemplateOptions,
                error: function( pData, err, message ) {
                    apex.debug(pData);
                    apex.util.delayLinger.finish( "templateOptions", function() {
                        lSpinner$.remove();
                    });
                    alert( apex.lang.getMessage("APEX.LTO.NO_OPTIONS_FOUND") );
                }
            });
        },
        quickEdit: function() {
            var self = this;

            if ( started ) {
                this.endDomLocate();
            } else {
                this.beginDomLocate( function( el ) {
                    var parts, urlParts,
                        url = getBuilderUrl( self.actions.quickEdit );
                    self.navigateInPageDesigner( self.currentApp, el.pageId, el.typeId, el.componentId, function() {
                        // if that fails navigate to correct place
                        if ( el.pageId !== self.currentPage ) {
                            urlParts = url.split( ":" );
                            parts = urlParts[ urlParts.length - 1 ].split( "," );
                            parts[1] = parts[3] = parts[4] = el.pageId;
                            urlParts[ urlParts.length - 1 ] = parts.join( "," );
                            url = urlParts.join(":");
                        }
                        url += "#" + el.typeId + ":" + el.componentId;
                        self.builderWindowUrl( url );
                    } );
                });
            }
        },

        themeRoller: function() {
            var self = this;
            apex.debug.info( "ThemeId: %s, BuilderSessionId: %s", self.themeId, self.builderSessionId );

            var toggle = function() {
                if ( apex.utr && apex.utr.opened ) {
                    $.universalThemeRoller("close");
                } else {
                    $.universalThemeRoller("open");
                }
            };

            if ( $.universalThemeRoller ) {
                toggle();
            } else {
                var onThemeRollerLoad = getThemeRollerInit( self.themeId, self.builderSessionId, toggle );
                // No need to load utr-base.js, it should have already been loaded by now.
                $.getScript( THEMEROLLER_BASE + 'jquery.universalThemeRoller.js', onThemeRollerLoad );
            }
        }
    };

    function fixToolbarWidth() {
        var o, tbWidth, windowWidth,
            dtb$ = $( "#apexDevToolbar"),
            direction = dtb$.css("direction") === "rtl" ? "right" : "left"; // when in RTL mode, the left CSS property

        o = {
            width: ""
        };
        if ( dtb$.hasClass("a-DevToolbar--top") || dtb$.hasClass("a-DevToolbar--bottom") ) {
            windowWidth = getWindowWidth();
            o.whiteSpace = "nowrap";  // clear element width to get desired width of ul content
            dtb$.css( o );
            // using width assuming no margin etc.
            tbWidth = dtb$.children( "ul" )[0].clientWidth + 4; // IE wants just a little extra to keep the buttons from wrapping
            if ( tbWidth > windowWidth ) {
                tbWidth = windowWidth;
            }
            o.whiteSpace = "wrap";
            o.width = tbWidth;
            o[direction] = (windowWidth - tbWidth) / 2; // position the offset in the center.
        } else {
            o[direction] = ""; // clear the offset and width
        }
        dtb$.css( o );
    }

    function updateButtons( ctx ) {
        $( "#apexDevToolbarApp" ).attr("title", ctx.appTitle ).find(".a-DevToolbar-buttonLabel" ).text( ctx.appTitle );
        $( "#apexDevToolbarPage" ).attr("title", ctx.pageTitle ).find(".a-DevToolbar-buttonLabel" ).text( ctx.pageTitle );
        $( "#apexDevToolbarDebug" ).attr("title", ctx.debugTitle ).find(".a-DevToolbar-buttonLabel" ).text( ctx.debugTitle );
        fixToolbarWidth();
    }

    function pushContext() {
        gContextStack.push( {} );
    }

    function setContext( toolbar$, wnd, components, windowMgmtMode, text, themeId ) {
        var that = Object.create( contextPrototype ),
            url = getBuilderUrl( getUrl( toolbar$.find( "#apexDevToolbarPage" ) ) ),
            parts = url.split( ":" )[7].split( "," );

        that.currentApp = parts[0];
        that.currentPage = parts[1];
        that.builderSessionId = url.split( ":" )[2];
        that.document = toolbar$[0].ownerDocument;
        that.window = wnd;
        that.actions = {
            home: getUrl( toolbar$.find( "#apexDevToolbarHome" ) ),
            app: getUrl( toolbar$.find( "#apexDevToolbarApp" ) ),
            page: getUrl( toolbar$.find( "#apexDevToolbarPage" ) ),
            session: getUrl( toolbar$.find( "#apexDevToolbarSession" ) ),
            viewDebug: getUrl( toolbar$.find( "#apexDevToolbarViewDebug" ) ),
            debug: getUrl( toolbar$.find( "#apexDevToolbarDebug" ) ),
            quickEdit: getUrl( toolbar$.find( "#apexDevToolbarQuickEdit" ) )
        };
        that.components = components;
        that.windowMgmtMode = windowMgmtMode;
        that.themeId = themeId;
        that.text = text;
        that.appTitle = toolbar$.find( "#apexDevToolbarApp" ).attr( "title" );
        that.pageTitle = toolbar$.find( "#apexDevToolbarPage" ).attr( "title" );
        that.debugTitle = toolbar$.find( "#apexDevToolbarDebug" ).attr( "title" );

        gContextStack[gContextStack.length - 1] = that;
        updateButtons( getContext() );
    }

    function popContext() {
        gContextStack.pop();
        updateButtons( getContext() );
    }

    function getContext() {
        var i;
        for ( i = gContextStack.length - 1; i >= 0; i-- ) {
            if ( gContextStack[i].currentApp !== undefined ) {
                return gContextStack[i];
            }
        }
        return null; // don't expect to get here
    }

    function openBuilderIfNeeded() {
        if ( !getBuilderInstance() ) {
            getContext().builderWindow("page");
            return true;
        } else {
            return false;
        }
    }

    apex.initNestedDevToolbar = function( toolbar$, wnd, components, windowMgmtMode, text, themeId ) {
        setContext( toolbar$, wnd, components, windowMgmtMode, text, themeId );
    };

    /*
     * Must be called from document ready handler
     */
    apex.initDevToolbar = function( components, windowMgmtMode, text, themeId ) {
        var focused = false,
            menuOpen = false,
            // this menu is hooked up to the Options menu button by id
            optionsMenu$ = $( "<div id='apexDevToolbarMenu'></div>" ).appendTo( "body" ),
            hideTimer = null,
            // These options are persisted in local storage
            autoHide = false,
            iconsOnly = false,
            displayPosition = "bottom";

        function saveOptions() {
            if ( storage.hasLocalStorageSupport() ) {
                // Would love to use JSON.stringify but need to support IE7 (sure there are libraries
                // that provide a back-fill for JSON but we have no other need)
                gDevToolbarLocalStore.setItem( "options", '{"autoHide":' + autoHide + ',"iconsOnly":' + iconsOnly + ',"displayPosition":"' + displayPosition + '"}' );
            }
        }

        function loadOptions() {
            var options;

            options = gDevToolbarLocalStore.getItem( "options" );
            if ( options ) {
                try {
                    options = $.parseJSON( options );
                    autoHide = options.autoHide ? options.autoHide : autoHide;
                    iconsOnly = options.iconsOnly ? options.iconsOnly : iconsOnly;
                    displayPosition = /^(top|left|right|bottom)$/.test(options.displayPosition) ? options.displayPosition : displayPosition;
                } catch ( ex ) {
                    // Ignore any exception. If someone has messed with the options no worries the next saveOptions will set things right
                }
            }
        }

        function updateGrid( show ) {
            var state = show ? "on" : "off";

            $( "#grid_debug_on" ).parent().toggle( !show );
            $( "#grid_debug_off" ).parent().toggle( show );
            $( document ).trigger( "apex-devbar-grid-debug-" + state );
            gDevToolbarSessionStore.setItem( "grid", state );
            // don't put call to fixToolbarWidth here because it then happens twice during page init
        }
        updateGrid( gDevToolbarSessionStore.getItem( "grid" ) === "on" );

        // if this page is in an iframe (or frame) don't show the toolbar but do let the toolbar on the top window handle this page
        if ( self.apex !== util.getTopApex() ) {
            if ( util.getTopApex().initNestedDevToolbar ) {
                util.getTopApex().initNestedDevToolbar( $( "#apexDevToolbar" ), window, components, windowMgmtMode, text, themeId );
            }
            // If TR exists on the parent window, register this TR instance as a child
            // This TR instance Will receive instructions to modify the style of the
            // document appropriately.
            if ( util.getTopApex().utr ) {
                window.apex.utr = {
                    nested: true
                };
                var onUTRLoad = function() {
                    if ( window.apex.utr.nest ) {
                        util.getTopApex().utr.nest && util.getTopApex().utr.nest( window.apex.utr );
                    } else {
                        setTimeout( onUTRLoad, 100 );
                    }
                };
                $.getScript( THEMEROLLER_BASE + "utr-base.js", onUTRLoad );
            }

            return;
        }

        var onThemeRollerLoad = getThemeRollerInit( themeId, getBuilderInstance(), function() { $.universalThemeRoller('open'); } );

        // utr-base must ALWAYS be loaded for proper iframe (read: modal) support.
        $.getScript( THEMEROLLER_BASE + 'utr-base.js', function(){
            // Load ThemeRoller scripts if it was already open.
            if ( gThemeRollerSessionStore.getItem( 'OPENED' ) === 'true' ) {
                $.getScript( THEMEROLLER_BASE + 'jquery.universalThemeRoller.js', onThemeRollerLoad );
            }
        });

        $( "#apexDevToolbarHome" ).click( function() {
            getContext().builderWindow( "home" );
        });

        $( "#apexDevToolbarApp" ).click( function() {
            getContext().builderWindow( "app" );
        });

        $( "#apexDevToolbarPage" ).click( function() {
            var ctx = getContext();

            // first try to tell the page designer what app and page
            ctx.navigateInPageDesigner( ctx.currentApp, ctx.currentPage, null, null, function() {
                // if that fails navigate to correct place
                ctx.builderWindow( "page" );
            } );
        });

        $( "#apexDevToolbarSession" ).click( function() {
            getContext().popup( "session" );
        });

        $( "#apexDevToolbarViewDebug" ).click( function() {
            getContext().popup( "viewDebug" );
        });

        $( "#apexDevToolbarDebug" ).click( function() {
            getContext().sameWindow( "debug" );
        });

        $( "#grid_debug_on" ).click( function() {
            updateGrid(true);
            fixToolbarWidth();
        });

        $( "#grid_debug_off" ).click( function() {
            updateGrid(false);
            fixToolbarWidth();
        } );

        $( "#apexDevToolbarQuickEdit" ).off( "click" ).click( function() {
            getContext().quickEdit();
        });

        $( "#apexDevToolbarThemeRoller" ).off( "click" ).click( function() {
            getContext().themeRoller();
        });

        optionsMenu$.menu({
            items: [
                { type: "toggle", label: text.autoHide, get: function () {
                    return autoHide;
                }, set: function ( v ) {
                    autoHide = v;
                    saveOptions();
                    $( "#apexDevToolbar" ).toggleClass( "a-DevToolbar--autoHide", autoHide );
                } },
                { type: "toggle", label: text.iconsOnly, get: function () {
                    return iconsOnly;
                }, set: function ( v ) {
                    iconsOnly = v;
                    saveOptions();
                    $( "#apexDevToolbar" ).toggleClass( "a-DevToolbar--iconsOnly", iconsOnly );
                    fixToolbarWidth();
                } },
                { type: "subMenu", label: text.display, menu: { items: [
                    {
                        type: "radioGroup",
                        get: function () {
                            return displayPosition;
                        },
                        set: function ( pValue ) {
                            displayPosition = pValue;
                            saveOptions();
                            $( "#apexDevToolbar" ).removeClass( "a-DevToolbar--top a-DevToolbar--left a-DevToolbar--bottom a-DevToolbar--right" );
                            $( "#apexDevToolbar" ).addClass( "a-DevToolbar--" + displayPosition );
                            fixToolbarWidth();
                        },
                        choices: [
                            { label: text.displayTop, value: "top" },
                            { label: text.displayLeft, value: "left" },
                            { label: text.displayBottom,  value: "bottom" },
                            { label: text.displayRight,  value: "right" }
                        ]
                    }
                ]}
                }
            ],
            beforeOpen: function() {
                menuOpen = true;
            }
        });

        loadOptions();

        $( "#apexDevToolbar" )
            .toggleClass( "a-DevToolbar--iconsOnly", iconsOnly )
            .toggleClass( "a-DevToolbar--autoHide", autoHide )
            .removeClass( "a-DevToolbar--top a-DevToolbar--left a-DevToolbar--bottom a-DevToolbar--right" )
            .addClass( "a-DevToolbar--" + displayPosition )
            .on( "focusin", function() {
                focused = true;
                menuOpen = false;
                $( this ).addClass( "is-active" );
            }).on( "focusout", function() {
                focused = false;
                if ( !menuOpen ) {
                    $( this ).removeClass( "is-active" );
                }
            }).on( "mouseenter", function() {
                if ( hideTimer !== null ) {
                    clearTimeout( hideTimer );
                    hideTimer = null;
                }
                $( this ).addClass( "is-active" );
            }).on( "mouseleave", function() {
                var self = this;
                hideTimer = setTimeout( function() {
                    hideTimer = null;
                    if ( !focused && !menuOpen ) {
                        $( self ).removeClass( "is-active" );
                    }
                }, 1000 );
            }).show();

        pushContext();
        setContext( $( "#apexDevToolbar" ), window, components, windowMgmtMode, text, themeId );

        $( window ).on( "apexwindowresized", function() {
            fixToolbarWidth();
        });

        $( document.body ).on( "dialogopen", function( event ) {
            pushContext(); // it may or may not be an APEX page, won't know till it loads but push a new context anyway
        } ).on( "dialogclose", function( event ) {
            popContext();
        } );

    };


})( apex.jQuery, apex.util, apex.navigation, apex.storage );

