/*!
 CSS Calendar Widget
 Copyright (c) 2015, 2016 Oracle and/or its affiliates. All rights reserved.
 */

/**
 * @fileOverview
 * The {@link apex.widget}.cssCalendar is used for the CSS Calendar widget of Oracle Application Express.
 **/

(function ( widget, util, server, $ ) {

    /**
     * @param {String} pRegionId  Region ID to identify APEX region for this widget.
     * @param {Object} [pOptions]
     * @param {String} [pLocale]
     * @param {String} [pPluginInitJavascript]
     *
     * @function cssCalendar
     * @memberOf apex.widget
     * */


    widget.cssCalendar = function ( pRegionId, pOptions, pLocale, pPluginInitJavascript ) {

        var FC   = $.fullCalendar;

        if ( !pOptions.isMobile ) {

            // *****************************************************************************
            // L I S T V I E W
            // *****************************************************************************

            var View = FC.View;
            var ListView;


            ListView = View.extend( {
                                        setRange: function ( range ) {
                                            var rangeSet = false;
                                            if ( gCalStorage ) {
                                                var lastViewStr = gCalStorage.getItem( "lastview" );
                                                var lastViewObj = false;
                                                if ( lastViewStr ) {
                                                    lastViewObj = $.parseJSON( lastViewStr );
                                                }
                                                if ( lastViewObj ) {
                                                    if ( lastViewObj.viewType != "list" ) {
                                                        if ( lastViewObj.viewStartDate &&
                                                             lastViewObj.viewEndDate &&
                                                             lastViewObj.viewIntervalUnit
                                                        ) {
                                                            if ( !gOptions.listViewAlwaysShowMonth ) {
                                                                range.intervalStart = FC.moment( lastViewObj.viewStartDate );
                                                                range.intervalEnd   = FC.moment( lastViewObj.viewEndDate );
                                                                range.start         = FC.moment( lastViewObj.viewStartDate );
                                                                range.end           = FC.moment( lastViewObj.viewEndDate );
                                                                range.intervalUnit  = lastViewObj.viewIntervalUnit;
                                                                rangeSet            = true;
                                                            }
                                                        }
                                                    } else {
                                                        rangeSet = true;
                                                    }
                                                }
                                            }
                                            if ( !rangeSet ) {
                                                range.intervalStart = FC.moment().startOf( gOptions.defaultView );
                                                range.intervalEnd   = FC.moment().endOf( gOptions.defaultView );
                                                range.start         = FC.moment().startOf( gOptions.defaultView );
                                                range.end           = FC.moment().endOf( gOptions.defaultView );
                                                range.intervalUnit  = gOptions.defaultView;
                                            }
                                            View.prototype.setRange.call( this, range );
                                            switch ( range.intervalUnit ) {
                                                case "day":
                                                    this.intervalDuration = moment.duration( 1, "days" );
                                                    break;
                                                case "week":
                                                    this.intervalDuration = moment.duration( 1, "weeks" );
                                                    break;
                                                default:
                                                    this.intervalDuration = moment.duration( 1, "months" );
                                            }

                                            switch ( range.intervalUnit ) {
                                                case "day":
                                                    this.title = this.start.format( "LL" );
                                                    break;
                                                case "week":
                                                    this.title = $.fullCalendar.formatRange(
                                                        range.start,
                                                        range.end.clone().subtract( 1, "days" ), // make inclusive by subtracting 1 ms
                                                        "ll",
                                                        ' \u2014 ' // emphasized dash
                                                    );
                                                    break;
                                                default:
                                                    this.title = this.start.format( this.opt( 'titleFormat' ) || this.computeTitleFormat() );
                                            }
                                        },


                                        renderEvents: function ( events ) {
                                            var t = this;

                                            var displayeventlist = [];
                                            var j                = 0;

                                            var tstart, tend;

                                            var html, dayLi, dayLiHead, eventHtml, evtUl, eventdisplay;

                                            var disDay, disDate, lurl, ltitle, lTimeFmt, description, classes;
                                            var allDay, startDate, endDate;
                                            var temp = "", i;

                                            if ( gOptions.timeFormatType == "12" ) {
                                                lTimeFmt = "ll" + " hh:mm A";
                                            } else if ( gOptions.timeFormatType == "24" ) {
                                                lTimeFmt = "ll" + " HH:mm";
                                            } else {
                                                lTimeFmt = "lll";
                                            }

                                            // Duplicate the list of events and use the duplicate to format and print ordered list of events
                                            // We need to duplicate so that we can have multiple-days event split into number of each day event for the diplaying purpose
                                            // Event start Monday to Friday, we will have to display the same event everyday, hence adding that on Tuesday, Wednesday and Thursday

                                            for ( i in events ) {
                                                displayeventlist[ j ] = $.extend( true, {}, events[ i ] );

                                                tstart = events[ i ].start.clone();
                                                if ( displayeventlist[ j ].end ) {
                                                    if ( displayeventlist[ j ].allDay ) {
                                                        tend = events[ i ].end.clone().subtract( 1, "days" );
                                                    } else {
                                                        tend = events[ i ].end.clone().subtract( 1, "seconds" );
                                                    }
                                                } else {
                                                    tend = null;
                                                }

                                                /* for multiple-days event, we need to make fake events for the sake of displaying the list
                                                 If an event start from Januray 12 - Janurary 15 / we need to display this event in 12/13/14/15
                                                 */
                                                displayeventlist[ j ].displayDay = events[ i ].start;
                                                while ( ( tend - tstart ) >= 1000 * 59 * 60 * 24 ) {
                                                    j      = j + 1;
                                                    tstart = tstart.add( 1, 'day' );
                                                    if ( !tstart.clone().startOf( "day" ).isAfter( tend ) ) {
                                                        displayeventlist[ j ]            = $.extend( true, {}, events[ i ] );
                                                        displayeventlist[ j ].displayDay = tstart.clone();
                                                    }
                                                }
                                                j = j + 1;
                                            }

                                            // Lets sort our duplicated list start from day 1 to the last of each month
                                            displayeventlist.sort( function ( a, b ) {
                                                var dateA = formatYYYYMMDDhhmmss( a.displayDay ) + a.title ;
                                                var dateB = formatYYYYMMDDhhmmss( b.displayDay ) + b.title ;
                                                if ( dateA < dateB ) {
                                                    return -1;
                                                } else if ( dateA > dateB ) {
                                                    return 1;
                                                } else {
                                                    return 0;
                                                }
                                            } );

                                            //Start displaying our sorted list
                                            html = $( apex.util.htmlBuilder().markup( "<ul" ).attr( "class", "fc-agendaList" ).markup( "></ul>" ).toString() );

                                            // scan through fake list of events and display them, but check only those of current month
                                            if ( displayeventlist.length === 0 ) {
                                                dayLi = $( apex.util.htmlBuilder()
                                                               .markup( "<li" ).attr( "class", "fc-agendalist" ).markup( ">" )
                                                               .markup( "<h3" ).attr( "class", "fc-agendaList-dayHeader ui-widget-header" ).markup( "></h3>" )
                                                               .markup( "<div>" ).content( apex.lang.getMessage( "APEX.IG.NO_DATA_FOUND" ) ).markup( "</div>" )
                                                               .markup( "</li>" ).toString() ).appendTo( html );
                                            }
                                            for ( i in displayeventlist ) {
                                                endDate = null;
                                                if (
                                                    ( displayeventlist[ i ].displayDay.isAfter( t.intervalStart ) && displayeventlist[ i ].displayDay.isBefore( t.intervalEnd ) ) ||
                                                    displayeventlist[ i ].displayDay.isSame( t.intervalStart, 'day' )
                                                ) {
                                                    if ( gOptions.weekEnds || displayeventlist[ i ].displayDay.format( "E" ) <= 5 ) {
                                                        disDay    = displayeventlist[ i ].displayDay.format( "dddd" );
                                                        disDate   = displayeventlist[ i ].displayDay.format( "LL" );
                                                        ltitle    = displayeventlist[ i ].title;
                                                        allDay    = displayeventlist[ i ].allDay;
                                                        startDate = displayeventlist[ i ].start.format( allDay ? "ll" : lTimeFmt );
                                                        if ( displayeventlist[ i ].end ) {
                                                            endDate = displayeventlist[ i ].end.clone().subtract( ( allDay ? 1 : 0 ), "day" ).format( allDay ? "ll" : lTimeFmt );
                                                        }
                                                        lurl        = (displayeventlist[ i ].url ? displayeventlist[ i ].url : "undefined");
                                                        description = (displayeventlist[ i ].description ? displayeventlist[ i ].description : "");
                                                        classes     = displayeventlist[ i ].className;

                                                        // if the events are from source, then pick the className from the source not from event object itself
                                                        if ( displayeventlist[ i ].source ) {
                                                            classes = classes.concat( displayeventlist[ i ].source.className );
                                                        }

                                                        if ( disDate != temp ) {
                                                            dayLi     = $( apex.util.htmlBuilder().markup( "<li" ).attr( "class", "fc-agendalist" ).markup( "></li>" ).toString() ).appendTo( html );
                                                            dayLiHead = $( apex.util.htmlBuilder()
                                                                               .markup( "<h3" ).attr( "class", "fc-agendaList-dayHeader ui-widget-header" ).markup( ">" )
                                                                               .markup( "<span" ).attr( "class", "fc-agendaList-day" ).markup( ">" )
                                                                               .content( disDay )
                                                                               .markup( "</span>" )
                                                                               .markup( "<span" ).attr( "class", "fc-agendaList-date" ).markup( ">" )
                                                                               .content( disDate )
                                                                               .markup( "</span>" )
                                                                               .markup( "</h3>" ).toString() ).appendTo( dayLi );
                                                            evtUl     = $( apex.util.htmlBuilder().markup( "<ul" ).attr( "class", "fc-agendaList" ).markup( "></ul>" ).toString() ).appendTo( dayLi );
                                                            temp      = disDate;
                                                        }

                                                        // if the event is all day , make sure you print that and not date and time
                                                        // otherwise do the opposite
                                                        eventHtml = apex.util.htmlBuilder()
                                                                        .markup( "<li" ).attr( "class", "fc-agendaList-item" ).markup( ">" );
                                                        if ( lurl != "undefined" ) {
                                                            eventHtml.markup( "<a" ).attr( "href", lurl );
                                                        } else {
                                                            eventHtml.markup( "<div" );
                                                        }
                                                        eventHtml.attr( "class", "fc-agendaList-event fc-eventlist " + classes.join( ' ' ) );
                                                        eventHtml.markup( ">" );
                                                        eventHtml.markup( "<div" ).attr( "class", "fc-event-time" ).markup( ">" );
                                                        if ( allDay && ( !endDate || !displayeventlist[ i ].end.clone().subtract( 1, "day" ).isAfter( displayeventlist[ i ].start ) ) ) {
                                                            eventHtml.markup( "<span" ).attr( "class", "fc-event-all-day" ).markup( ">" ).content( (this.opt( 'allDayText' ) ? this.opt( 'allDayText' ) : 'all-day') ).markup( "</span>" );
                                                        } else {
                                                            if ( endDate ) {
                                                                eventHtml.markup( "<span" ).attr( "class", "fc-event-start-time" ).markup( ">" ).content( startDate ).markup( "</span>" );
                                                                eventHtml.markup( "<span" ).attr( "class", "fc-event-end-time" ).markup( ">" ).content( endDate ).markup( "</span>" );
                                                            } else {
                                                                eventHtml.markup( "<span" ).attr( "class", "fc-event-start-time" ).markup( ">" ).content( startDate ).markup( "</span>" );
                                                            }
                                                        }
                                                        eventHtml.markup( "</div>" );
                                                        eventHtml.markup( "<div" ).attr( "class", "fc-agendaList-eventDetails" ).markup( ">" );
                                                        eventHtml.markup( "<div" ).attr( "class", "fc-eventlist-title" ).markup( ">" ).content( ltitle ).markup( "</div>" );
                                                        if ( description ) {
                                                            eventHtml.markup( "<div" ).attr( "class", "fc-eventlist-desc" ).markup( ">" ).content( description ).markup( "</div>" );
                                                        }
                                                        eventHtml.markup( "</div>" );
                                                        if ( lurl != "undefined" ) {
                                                            eventHtml.markup( "</a>" );
                                                        } else {
                                                            eventHtml.markup( "</div>" );
                                                        }
                                                        eventHtml.markup( "</li>" );
                                                        eventdisplay = $( eventHtml.toString() ).appendTo( evtUl );
                                                        this.trigger( 'eventRender', displayeventlist[ i ], displayeventlist[ i ], eventdisplay );
                                                    }
                                                }
                                            }
                                            View.prototype.renderEvents.call( this, events );
                                            this.el.html( html );
                                            this.trigger( 'eventAfterAllRender' );
                                        }
                                    } );

            FC.views.list = ListView; // register our class with the view system
        }

        // *****************************************************************************
        // C A L E N D A R - I N I T I A L I Z A T I O N
        // *****************************************************************************

        var gOptions, gRegion$, gCalendar$, gLocale, gAllDay, gCalStorage, gLastView;
        var gDayNamesShort, gLastViewResponsive = false;

        var gEventId = 0;

        // this object holds state information for keyboard support
        var keyBoardInfo = {
            keyboardActive : false,
            focusEvents    : false,
            selectorEvent  : false,
            browseMode     : "CALENDAR",
            dayEvents      : [],
            activeDayEvent : false,
            SELECTORID     : "com_oracle_apex_selectorevent$",
            cssClass       : 'fc-helper',
            KEY_PREV       : 37,
            KEY_NEXT       : 39,
            MONTH_VIEW     : "month",
            WEEK_VIEW      : "agendaWeek",
            DAY_VIEW       : "agendaDay",
            LIST_VIEW      : "list",
            viewChangeByKey: false
        };

        _init();

        function _init() {

            gRegion$   = $( "#" + pRegionId, apex.gPageContext$ );
            gCalendar$ = $( "#" + pRegionId + "_calendar", apex.gPageContext$ );

            // if there is no region container, add one on the fly. It's necessary for our refresh mechanism
            if ( gRegion$.length === 0 ) {
                gRegion$ = gCalendar$.wrap(
                    apex.util.htmlBuilder().markup( "<div" ).attr( "id", pRegionId ).markup( "></div>" ).toString()
                );
            }

            gLocale  = $.fullCalendar.defaults;
            gOptions = $.extend( {
                                     enableDragAndDrop: false,
                                     pageItems        : ""
                                 },
                                 pOptions );

            gOptions = $.extend( gOptions, gLocale );

            gOptions.timeFormat = {
                month : "",
                list  : "",
                agenda: ""
            };

            // user will choose own settings 12 or 24

            if ( gOptions.timeFormatType === "24" ) {
                gOptions.axisFormat = "H:mm";
                gOptions.timeFormat = "H:mm";
                gOptions.views      = {
                    month : {
                        timeFormat: "HH"
                    },
                    list  : {
                        timeFormat: "H:mm"
                    },
                    agenda: {
                        timeFormat: "H:mm"
                    }
                };
            } else if ( gOptions.timeFormatType === "12" ) {
                gOptions.axisFormat = "h(:mm)a";
                gOptions.timeFormat = "h:mm a";
                gOptions.views      = {
                    month : {
                        timeFormat: "ha"
                    },
                    list  : {
                        timeFormat: "h:mm a"
                    },
                    agenda: {
                        timeFormat: "h:mm a"
                    }
                };
            } else {
                gOptions.axisFormat = "";
                gOptions.timeFormat = "";
                gOptions.views      = {
                    month : {
                        timeFormat: ""
                    },
                    list  : {
                        timeFormat: ""
                    },
                    agenda: {
                        timeFormat: ""
                    }
                };
            }

            // check if we need all-day slot
            gAllDay             = !!(gOptions.apexWebLink || gOptions.googleId);
            gOptions.allDayText = "all-day"; // english text; other langs will be translated.

            // make some updates from the defaults values
            gOptions.columnFormat = { month: "dddd" };

            //make sure if no drag&drop, it is totally disabled
            if ( gOptions.enableDragAndDrop ) {
                // adjust the resize depending on drag and drop and end column
                if ( gOptions.enableResizing ) {
                    gOptions.eventStartEditable    = true;
                    gOptions.eventDurationEditable = true;
                    gOptions.editable              = true;
                } else {
                    gOptions.editable              = true;
                    gOptions.eventStartEditable    = true;
                    gOptions.eventDurationEditable = false;
                }
            } else {
                gOptions.eventStartEditable    = false;
                gOptions.eventDurationEditable = false;
                gOptions.editable              = false;
            }

            gOptions.keyBoardSupport = true;

            // The session storage to keep track of the calendar view, date changes
            gCalStorage = apex.storage.getScopedSessionStorage( {
                                                                    prefix   : "ORA_WWV_apex.Calendar",
                                                                    useAppId : true,
                                                                    usePageId: true,
                                                                    regionId : pRegionId
                                                                } );

            //if the item already exists, make sure we know where the user was (view and date)
            gLastView = gCalStorage.getItem( "lastview" );

            //if item lastview exists then try to parse it
            var currentDate = FC.moment();   //the date we want calendar to start from
            if ( gLastView ) {
                try {
                    gLastView = $.parseJSON( gLastView );
                    // is this a good json object ? then check if it has the correct moment objects
                    if ( gLastView !== null ) {
                        if ( moment( gLastView.viewStartDate ).isValid() ) {
                            currentDate = gLastView.viewStartDate;
                        }
                    }
                } catch ( ex ) {
                    currentDate = FC.moment();     //make sure the start date is today
                    gLastView = null;         //make sure this is set to null
                }
            }

            // previous and next keys are different in RTL
            if ( gOptions.isRTL ) {
                keyBoardInfo.KEY_PREV = 39;
                keyBoardInfo.KEY_NEXT = 37;
            }

            // adjust abbrev. months array to locale
            if ( FC.moment( "2016-03-13" ).format( "e" ) == "0" ) { // en
                gDayNamesShort = apex.locale.getAbbrevDayNames().slice( 0 );
            } else {
                gDayNamesShort = apex.locale.getAbbrevDayNames().slice( 0, 6 );
                gDayNamesShort.unshift( apex.locale.getAbbrevDayNames().slice( 6 ) );
            }

            // setup options to initialize FullCalendar object
            var calOptions = {
                theme                    : true,
                lang                     : pLocale,
                header                   : JSON.parse( gOptions.calendarHeader ),
                monthNames               : gOptions.monthNames,
                monthNamesShort          : apex.locale.getAbbrevMonthNames(),
                dayNames                 : gOptions.dayNames,
                dayNamesShort            : gDayNamesShort,
                buttonText               : gOptions.buttonText,
                timeFormat               : gOptions.timeFormat,
                views                    : gOptions.views,
                displayEventTime         : false,
                columnFormat             : gOptions.columnFormat,
                defaultView              : gOptions.defaultView,
                titleFormat              : gOptions.titleFormat,
                axisFormat               : gOptions.axisFormat,
                firstDay                 : gOptions.firstDayOfWeek,
                isRTL                    : gOptions.isRTL,
                editable                 : gOptions.editable,
                eventStartEditable       : gOptions.eventStartEditable,
                eventDurationEditable    : gOptions.eventDurationEditable,
                nextDayThreshold         : '00:00:00',
                weekends                 : gOptions.weekEnds,
                eventLimit               : ( parseInt( gOptions.eventLimit ) < 2 ? 2 : parseInt( gOptions.eventLimit ) ),
                defaultTimedEventDuration: '01:00:00',
                aspectRatio              : _calcAspectRatio(),
                scrollTime               : lPadTime( gOptions.startingHour ),
                minTime                  : ( gOptions.minTime ? gOptions.minTime : "00:00:00" ),
                maxTime                  : ( gOptions.maxTime ? gOptions.maxTime : "23:59:59" ),
                allDaySlot               : gAllDay,
                allDayDefault            : gAllDay,
                googleCalendarApiKey     : gOptions.googleApiKey,
                defaultDate              : currentDate,
                eventSources             : [ {
                    events          : _getEvents,
                    editable        : gOptions.editable,
                    startEditable   : gOptions.eventStartEditable,
                    durationEditable: gOptions.eventDurationEditable,
                    apexEventSource : "SQL"
                }, {
                    googleCalendarId: gOptions.googleId,
                    className       : "fc-apex-events-gcal",
                    apexEventSource : "EXT_JSON"
                }, {
                    events         : _getWebserviceEvents,
                    editable       : false,
                    className      : "fc-apex-events-webservice",
                    apexEventSource: "EXT_GOOGLE"
                } ],
                eventResize              : _eventResize,
                eventDrop                : _eventDrop,
                eventDragStart           : _eventDrag,
                eventClick               : _eventClick,
                windowResize             : _windowResize,
                select                   : _calSelect,
                slotEventOverlap         : false,
                defaultEventMinutes      : 60,
                selectable               : true,
                selectHelper             : true,
                fixedWeekCount           : false,
                eventRender              : _eventRender,
                viewRender               : function ( view, element ) {
                    // when there's no "last view", deactivate Keyboard Mode
                    if ( !gCalStorage || !gCalStorage.getItem( "lastview" ) ||
                         $.parseJSON( gCalStorage.getItem( "lastview" ) ).viewType != view.name ) {
                        leaveKeyboardMode();
                    }
                    // when view changed by clicking a button, deactivate Keyboard Mode
                    if ( !keyBoardInfo.viewChangeByKey ) {
                        leaveKeyboardMode();
                    } else {
                        keyBoardInfo.viewChangeByKey = false;
                    }
                    // Responsive View change can only be towards "list" view
                    if ( view.name != "list" ) {
                        gLastViewResponsive = false;
                    }

                    // set new view in local storage
                    gCalStorage.setItem( "lastview", JSON.stringify( {
                                                                         viewType        : view.name,
                                                                         viewStartDate   : view.intervalStart,
                                                                         viewEndDate     : view.intervalEnd,
                                                                         viewIntervalUnit: view.intervalUnit
                                                                     } ) );

                    // fire event for dynamic action
                    apex.event.trigger(
                        gRegion$,
                        "apexcalendarviewchange",
                        {
                            "viewType" : view.name,
                            "startDate": formatYYYYMMDDhhmmss( view.intervalStart ),
                            "endDate"  : formatYYYYMMDDhhmmss( view.intervalEnd.clone().subtract( 1, "second" ) )
                        }
                    );

                    // patch jQuery fullcalendar markup to be consumable by screenreaders 
                    //

                    if ( view.name === "month" || view.name === "agendaWeek" || view.name === "agendaDay" ) {
                        $( ".fc-view > table", gCalendar$ ).attr( "role", "presentation" );
                        $( ".fc-view > table > tbody.fc-body table", gCalendar$ ).attr( "role", "presentation" );
                        $( ".fc-view > table > thead.fc-head table", gCalendar$ ).attr( "role", "presentation" );
                        $( ".fc-view > table > thead.fc-head table > thead > tr > th ", gCalendar$ ).attr( "scope", "col" );
                    }

                    if ( view.name === "agendaWeek" || view.name === "month" ) {
                        $( ".fc-view > table > thead.fc-head table", gCalendar$ ).each( function( i, o ) {
                            $( o ).attr( 
                                "summary", 
                                $( "thead th:not(.fc-axis)", $( o ) ).first().text() +  
                                    " - " + 
                                    $( "thead th:last-child()", $( o ) ).text() 
                            ); 
                        } );
                    }
                    if ( view.name === "agendaDay" ) {
                        $( ".fc-view > table > thead.fc-head table", gCalendar$ ).each( function( i, o ) {
                            $( o ).attr( 
                                "summary", 
                                $( "thead th:not(.fc-axis)", $( o ) ).first().text()  
                            ); 
                        } );
                    }

                    if ( view.name === "month" ) {
                        $( ".fc-view > table > tbody.fc-body table", gCalendar$ ).each( function ( i, o ) {
                            $( o ).attr( 
                                "summary", 
                                $( "thead td:nth-child(1)", $( o ) ).attr( "data-date" ) + 
                                    " - " + 
                                    $( "thead td:last-child()", $( o ) ).attr( "data-date" ) 
                            );
                        } );
                    }


                    if ( !gOptions.isMobile ) {
                        if ( gOptions.keyBoardSupport ) {
                            $( ".fc-view,.fc-popover", gCalendar$ ).off( "keydown", viewEventHandler );
                            $( ".fc-view,.fc-popover", gCalendar$ ).on( "keydown", viewEventHandler );
                        }
                    }
                },
                eventAfterAllRender      : function ( view ) {
                    $( ".fc-prev-button", gCalendar$ ).attr( "alt", apex.lang.getMessage( "APEX.GV.PREV_PAGE" ) );
                    $( ".fc-next-button", gCalendar$ ).attr( "alt", apex.lang.getMessage( "APEX.GV.NEXT_PAGE" ) );
                    $( "div.fc-view-container div.fc-view", gCalendar$ ).attr( "tabindex", "0" );
                    if ( view.name != "list" ) {
                        $( "div.fc-view-container div.fc-view .fc-event", gCalendar$ ).attr( "tabindex", "-1" );
                    }
                    if ( gOptions.isRTL ) {
                        $( "button.fc-list-button", gCalendar$ ).css( "margin-right", "10px" );
                    } else {
                        $( "button.fc-list-button", gCalendar$ ).css( "margin-left", "10px" );
                    }
                }
            };

            if ( calOptions.defaultView == "week" ) { calOptions.defaultView = "agendaWeek"; }
            if ( calOptions.defaultView == "day" ) { calOptions.defaultView = "agendaDay"; }

            if ( $.isFunction( pPluginInitJavascript ) ) {
                var changedCalOptions = pPluginInitJavascript( calOptions );
                if ( changedCalOptions ) {
                    if ( changedCalOptions.dataFilter && $.isFunction( changedCalOptions.dataFilter ) ) {
                        gOptions.dataFilter       = changedCalOptions.dataFilter;
                    }
                    if ( changedCalOptions.endDateExclusive ) {
                        gOptions.endDateExclusive = true;
                    }
                    if ( changedCalOptions.disableKeyboardSupport ) {
                        gOptions.keyBoardSupport  = false;
                    }
                    if ( changedCalOptions.listViewAlwaysShowMonth ) {
                        gOptions.listViewAlwaysShowMonth = changedCalOptions.listViewAlwaysShowMonth;
                    }
                    calOptions = changedCalOptions;
                }
            }

            if ( !gOptions.isMobile ) {
                gCalendar$.fullCalendar( calOptions );
            } else {
                server.plugin( gOptions.ajaxIdentifier, {
                    x01      : gOptions.defaultView, /* action */
                    x02      : gOptions.startDate, /* start date */
                    x03      : gOptions.endDate, /* end date */
                    x09      : gOptions.curDate, /* current date or Today's date */
                    pageItems: gOptions.pageItems
                }, {
                   dataType     : "html",
                   refreshObject: gRegion$,
                   success      : function ( pData ) {
                       apex.jQuery( gRegion$, apex.gPageContext$ ).html( pData ).trigger( "create" );
                   }
               } );

            }

            if ( !gOptions.isMobile ) {
                if ( gOptions.keyBoardSupport ) {
                    $( ".fc-view,.fc-popover", gCalendar$ ).off( "keydown", viewEventHandler );
                    $( ".fc-view,.fc-popover", gCalendar$ ).on( "keydown", viewEventHandler );
                }
            }

            widget.util.onVisibilityChange( gRegion$[0], function( show ) {
                if ( show ) {
                    gCalendar$.fullCalendar( "render" );
                    gCalendar$.fullCalendar( "rerenderEvents" );
                }
            } );

            gRegion$.on( "apexrefresh", function () {
                _refresh();
            } );

            gCalendar$.on( "apexafterclosedialog", function () {
                // refresh your calendar region, do you have a refresh method, if so...
                _refresh();
            } );

            // -----------------------------------------------------------------
            // Register the Calendar with the region interface (feature #1794)
            //
            if ( !gOptions.isMobile ) {
                apex.region.create( pRegionId, {
                    type: "CssCalendar",
                    refresh: function() {
                        _refresh();
                    },
                    focus: function() {
                        gCalendar$.find("div.fc-view").focus();
                    },
                    widget: function() {
                        return gCalendar$;
                    }
                });
            } else {
                apex.region.create( pRegionId, {
                    type: "CssCalendar",
                    refresh: function() {
                        _refresh();
                    },
                    focus: function() {
                        gRegion$.focus();
                    }
                });
            }

            // make sure the user goes back to his previous view
            if ( gLastView !== null ) {
                if ( moment( gLastView.viewStartDate ).isValid() ) {
                    if ( gLastView.viewType === 'month' ||
                         ( gLastView.viewType === 'agendaWeek' && gOptions.showTime ) ||
                         ( gLastView.viewType === 'agendaDay' && gOptions.showTime ) ||
                         gLastView.viewType === 'list'
                    ) {
                        gCalendar$.fullCalendar( 'changeView', gLastView.viewType );
                    }
                }
            }
        }

        function viewEventHandler( e ) {
            var view = gCalendar$.fullCalendar( "getView" );
            if ( keyBoardInfo.keyboardActive ) {
                switch ( view.name ) {
                    case "month":
                    case "agendaWeek":
                    case "agendaDay":
                        processKeyEvents( view, e );
                        break;
                }
            } else {
                if ( view.name != "list" && ( 
                    $( ".fc-view", gCalendar$ ).is( ":focus" ) || 
                    $( ".fc-popover", gCalendar$ ).is( ":focus" )  
                ) && (
                    e.which == 37 ||
                    e.which == 39 ||
                    e.which == 40 ||
                    e.which == 38 ||
                    e.which == 13 ||
                    e.which == 34 ||
                    e.which == 33 ||
                    e.which == 32)
                ) {
                    e.preventDefault();
                    initKeyboardMode( view );
                } 
            }
        }

        // *****************************************************************************
        // A J A X - C A L L S
        // *****************************************************************************

        // Populates the FullCalendar with new events by issuing an AJAX request to the server
        function _getEvents( pStart, pEnd, pTimezone, pCallback ) {
            // make ajax call, to get events objects
            server.plugin( gOptions.ajaxIdentifier,
                           {
                               x01      : "GET", /* action */
                               x02      : formatYYYYMMDDhhmmss( pStart ),
                               x03      : formatYYYYMMDDhhmmss( pEnd ),
                               pageItems: gOptions.pageItems
                           }, {
                               refreshObject: gRegion$,
                               success      : function ( pData ) {
                                   var theData, changedData;
                                   theData = makeEndDateInclusive( pData );
                                   if ( $.isFunction( gOptions.dataFilter ) ) {
                                       changedData = gOptions.dataFilter( theData );
                                       if ( changedData ) {
                                           theData = changedData;
                                       }
                                   }
                                   pCallback( theData );
                               }
                           } );
        }


        // Populates the Fullcalendar with new events from apex webservice if used
        function _getWebserviceEvents( pStart, pEnd, pTimezone, pCallback ) {
            if ( gOptions.apexWebLink ) {
                $.ajax( {
                            url     : gOptions.apexWebLink,
                            dataType: 'json',
                            jsonp   : false,
                            success : function ( data ) {
                                var theData, changedData;
                                theData = makeEndDateInclusive( adjustAllDayForJsonEvents (data.items ) );
                                if ( $.isFunction( gOptions.dataFilter ) ) {
                                    changedData = gOptions.dataFilter( theData );
                                    if ( changedData ) {
                                        theData = changedData;
                                    }
                                }
                                pCallback( theData );
                            }
                        } );
            } else {
                pCallback( {} );
            }
        }

        // Issues an AJAX call to update the event on the server in case a user dragged or resized it
        function _changeEvent( pEvent, pDurationDelta, pRevertFunction ) {
            if ( pEvent.source.apexEventSource === "SQL" ) {
                server.plugin(
                    gOptions.ajaxIdentifier,
                    {
                        x01      : "CHANGE",
                        x02      : formatYYYYMMDDhhmmss( pEvent.start ),
                        x03      : ((pEvent.end !== null) ? formatYYYYMMDDhhmmss( subtractDayIfRequired( pEvent ) ) : null ),
                        x04      : pEvent.id,
                        x05      : pEvent.checksum,
                        pageItems: gOptions.pageItems
                    },
                    {
                        refreshObject: gRegion$,
                        success      : function ( data ) {
                            apex.event.trigger(
                                gRegion$,
                                'apexcalendardragdropfinish',
                                data
                            );
                        },
                        error        : function ( pjqXHR, pTextStatus, pErrorThrown ) {
                            // Revert drag and drop or resize operation if an error occurred on the backend
                            apex.event.trigger(
                                gRegion$,
                                'apexcalendardragdroperror',
                                pErrorThrown
                            );
                            pRevertFunction();
                        }
                    }
                );
            } else {
                pRevertFunction();
            }
        }

        function getUrlAndRedirectAjax( pStart, pEnd ) {
            server.plugin( gOptions.ajaxIdentifier, {
                x01: "PREPARE_URL", /* action */
                x02: formatYYYYMMDDhhmmss( pStart ),
                x03: formatYYYYMMDDhhmmss( pEnd ),
                x06: gOptions.createLink
            }, {
                success: function ( pData ) {
                    if ( pData.url ) {
                        apex.navigation.redirect( pData.url );
                    } else {
                        apex.debug( pData );
                    }
                }
            } );
        }

        // *****************************************************************************
        // H E L P E R - F U N C T I O N S
        // *****************************************************************************
        //

        function getNextId() {
            return "apex.widget.cssCalendar.Id_" + ( ++gEventId );
        }

        function getMoreLink( pDate ) {
            var lDateCell$ = $("div.fc-content-skeleton td[data-date=" + pDate + "]");
            var lDateIndex = lDateCell$.index() ;
        
            var lOffsets = [];
            var lSkips   = [];
        
            var lMoreLink$;
            var lStopSearch = false;
        
            function incrementOffset( idx ) {
                if ( lOffsets[ idx ] ) {
                    lOffsets[ idx ]++;
                } else {
                    lOffsets[ idx] = 1;
                }
            }
        
            function getOffset( idx ) {
                return lOffsets[ idx ] || 0;
            }
        
            lDateCell$.closest( "table" ).find( "tbody > tr" ).each( function( pRowIndex ) {
                var lthisRowOffset = getOffset( pRowIndex ),
                    lthisColNumber = -1,
                    i, j, k;
        
                apex.debug.trace( "Row Number:" + pRowIndex );
        
                if ( lStopSearch === false ) {
                    $( this ).children().each( function( pColIndex ) {
                        var lAttrRowspan = $( this ).attr( "rowspan" ),
                            lAttrColspan = $( this ).attr( "colspan" ) || 1,
                            lCol    = this;
        
                        apex.debug.trace( "Row Number:" + pColIndex );
        
                        for ( i = 0; i < lAttrColspan; i++) {
                            if ( $( lCol ).css( "display" ) !== "none" ) {
                                lthisColNumber++;
                                apex.debug.trace( "Col Number:" + lthisColNumber + " - Offset:" + lthisRowOffset );
        
                                if ( lthisColNumber + lthisRowOffset >= lDateIndex ) {
                                    if ( lSkips [ pRowIndex ] ) { return false; }
        
                                    if ( $( lCol ).find( "a.fc-more" ).length > 0 ) {
                                        apex.debug.trace( "found the link." );
                                        lMoreLink$ = $( lCol ).find( "a.fc-more" );
                                        lStopSearch = true;
                                    }
        
                                    if ( lAttrRowspan > 1 ) {
                                        for ( j = 1; j < lAttrRowspan; j++ ) {
                                            lSkips[ pRowIndex + j ] = true;
                                        }
                                    }
        
                                    return false;
                                }
        
                                if ( lAttrRowspan > 1 ) {
                                    for ( k = 1; k < lAttrRowspan; k++ ) {
                                        incrementOffset( pRowIndex + k );
                                    }
                                }
                            }
                        }
                    } );
                } 
            });
        
            return lMoreLink$;
        }
        
        function formatYYYYMMDD( pMoment ) {
            return ("0000" +   pMoment.year ( )       ).substr( -4 ) +
                   ("00"   + ( pMoment.month( ) + 1 ) ).substr( -2 ) +
                   ("00"   +   pMoment.date ( )       ).substr( -2 );
        }

        function formatYYYYMMDDhhmmss( pMoment ) {
            return formatYYYYMMDD( pMoment ) +
                   ("00"   + pMoment.hour  ( ) ).substr( -2 ) +
                   ("00"   + pMoment.minute( ) ).substr( -2 ) +
                   ("00"   + pMoment.second( ) ).substr( -2 );
        }

        function subtractDayIfRequired( pEvent ) {

            if ( pEvent.end !== null && pEvent.allDay && !gOptions.endDateExclusive ) {
                return pEvent.end.clone().subtract( 1, "days" );
            } else {
                return pEvent.end;
            }
        }

        function adjustAllDayForJsonEvents( pEventArray ) {
            var i;
            if ( !gOptions.showTime ) {
                for ( i = 0; i < pEventArray.length; i++ ) {
                    pEventArray[ i ].allDay = true;
                }
            }
            return pEventArray;
        }

        function makeEndDateInclusive( pEventArray ) {
            var i;
            if ( !gOptions.endDateExclusive ) {
                for ( i = 0; i < pEventArray.length; i++ ) {
                    if ( pEventArray[ i ].end ) {
                        pEventArray[ i ].end = moment( pEventArray[ i ].end, "YYYY-MM-DD[T]HH:mm:ss" ).add( ( pEventArray[ i ].allDay ? "1" : "0" ), "days" ).format( "YYYY-MM-DD[T]HH:mm:ss" );
                    }
                }
            }
            return pEventArray;
        }

        function lPadTime( pTime ) {
            return ( pTime <= 9 ? "0" + pTime + ":00:00" : "" + pTime + ":00:00");
        }

        // Called by the APEX refresh event to refetch events
        function _refresh() {
            if ( !gOptions.isMobile) {
                gCalendar$.fullCalendar( 'refetchEvents' );
                if ( keyBoardInfo.keyboardActive === true && keyBoardInfo.browseMode === "EVENTS" ) {
                    leaveKeyboardMode();
                }
            } else {
                apex.widget.cssCalendar.refreshMobileCalendar();
            }
        }

        function _eventRender( event, element ) {
            if ( !event.id || event.id === "" ) {
                event.id = getNextId();
            }
            element.attr( "data-id", event.id );
            if ( event.source ) {
                if ( event.source.apexEventSource == "SQL" ) {
                    element.find( 'span.fc-title' ).html( element.find( 'span.fc-title' ).text() );
                    element.find( 'div.fc-title' ).html( element.find( 'div.fc-title' ).text() );
                    element.find( 'div.fc-eventlist-title' ).html( element.find( 'div.fc-eventlist-title' ).text() );
                    element.find( 'div.fc-eventlist-desc' ).html( element.find( 'div.fc-eventlist-desc' ).text() );
                }
            }
            if ( gOptions.mouseHoverOver ) {
                _setTooltip( element, event );
            }
        }

        function _calcAspectRatio( ) {
            var wWidth         = $( window ).width( )  ,
                wHeight        = $( window ).height( ) ,
                rWidth         = gRegion$.width( )     ,
                rHeight        = gRegion$.height( )    ,
                rAspectRatio   = Math.round( ( rWidth / rHeight ) * 10 ) / 10,
                wAspectRatio   = Math.round( ( wWidth / wHeight ) * 10 ) / 10,
                theAspectRatio = rAspectRatio;


            if ( wAspectRatio < 1 && rAspectRatio > 1 ) { theAspectRatio = wAspectRatio; }
            else if ( wAspectRatio > 1 && rAspectRatio < 1 ) { theAspectRatio = 1; }

            if ( theAspectRatio < 1 ) { theAspectRatio = 1; }
            else if ( theAspectRatio > 2.6 ) { theAspectRatio = 2.6; }

            return theAspectRatio;
        }

        function _windowResize( view ) {
            var lWidth = $( window ).width();
            if ( view.name === "month" || view.name === "agendaWeek" ) {
                if ( $( window ).width() < 700 ) {
                    gLastViewResponsive = view.name;
                    gCalendar$.fullCalendar( "changeView", "list" );
                    gCalendar$.fullCalendar( "option", "aspectRatio", _calcAspectRatio( ) );
                }
            } else if ( view.name === "list" ) {
                if ( lWidth > 700 && gLastViewResponsive ) {
                    gCalendar$.fullCalendar( "changeView", gLastViewResponsive );
                    gCalendar$.fullCalendar( "option", "aspectRatio", _calcAspectRatio( ) );
                    gLastViewResponsive = false;
                }
            }
        }

        function _eventDrop( pEvent, pDurationDelta, pRevertFunction ) {
            if ( !apex.event.gCancelFlag ) {
                _changeEvent( pEvent, pDurationDelta, pRevertFunction );
            } else {
                pRevertFunction();
            }
        }

        function _eventDrag( pEvent ) {
            apex.event.trigger(
                gRegion$,
                'apexcalendardragdropstart',
                pEvent
            );
        }

        // Issues an AJAX call to update the event on the server in case a user changed the end date of an event
        function _eventResize( pEvent, pDurationDelta, pRevertFunction ) {
            _changeEvent( pEvent, pDurationDelta, pRevertFunction );
        }

        function _eventClick( pEvent, pJsEvent, pView ) {
            var lEvent;

            if ( pEvent.id == keyBoardInfo.SELECTORID ) {
                _calSelect( keyBoardInfo.selectorEvent.start.clone(), keyBoardInfo.selectorEvent.end.clone(), gAllDay );
            } else {
                if ( pEvent.url ) {
                    apex.navigation.redirect( pEvent.url );
                } else {
                    lEvent = $.extend( true, {}, pEvent );
                    lEvent.end = ( lEvent.end !== null ? subtractDayIfRequired( lEvent ) : null );

                    apex.event.trigger(
                        gRegion$,
                        "apexcalendareventselect",
                        {
                            "event": lEvent
                        }
                    );
                }
            }
            return false;
        }

        function _calSelect( pStart, pEnd, allDay ) {
            var lFormat;

            if ( gCalendar$.fullCalendar( "getView" ).name === "month" ) {
                pEnd.subtract(1, 'minutes');
                lFormat = formatYYYYMMDD;
            } else {
                lFormat = formatYYYYMMDDhhmmss;
            }
            if ( gOptions.createLink && gOptions.createLink !== "" ) {
                // For the CREATE LINK we add the selected START and END dates in JS code
                // if page checksum is enabled we need the server to recalculate the URL checksum
                getUrlAndRedirectAjax( pStart, pEnd );
            } else {
                // when no edit link is defined, fire event
                apex.event.trigger(
                    gRegion$,
                    "apexcalendardateselect",
                    {
                        "newStartDate": lFormat( pStart ),
                        "newEndDate"  : lFormat( pEnd )
                    }
                );
            }

            /* this is necessary once we have dialogs, because we wont leave the current page */
            gCalendar$.fullCalendar( "unselect" );
            gCalendar$.fullCalendar( "rerenderEvent" );
        }

        function eventEscaping( pEvent ) {
            return ( !(pEvent.source && pEvent.source.apexEventSource == 'SQL') );
        }

        function _setTooltip( element, pEvent ) {
            var dateDisplay, tooltip, timeFormatMask;
            if ( gOptions.timeFormatType == "24" ) { timeFormatMask = "ll HH:mm";}
            else if ( gOptions.timeFormatType == "12" ) { timeFormatMask = "ll hh:mm A";}
            else { timeFormatMask = "lll";}

            if ( pEvent.id != keyBoardInfo.SELECTORID && pEvent.title && pEvent.start ) {
                if ( pEvent.end ) {
                    if ( pEvent.allDay ) {
                        if ( pEvent.end.clone().subtract( 1, "days" ).isBefore( pEvent.start ) ) {
                            dateDisplay = $.fullCalendar.formatRange( pEvent.start, pEvent.end.clone(), 'll' );
                        } else {
                            dateDisplay = $.fullCalendar.formatRange( pEvent.start, subtractDayIfRequired( pEvent ), 'll' );
                        }
                    } else {
                        dateDisplay = $.fullCalendar.formatRange( pEvent.start, pEvent.end, timeFormatMask );
                    }
                } else {
                    if ( pEvent.allDay ) {
                        dateDisplay = pEvent.start.format( 'll' );
                    } else {
                        dateDisplay = pEvent.start.format( timeFormatMask );
                    }
                }

                tooltip =
                    '<span class="tt-title">' +
                    ( eventEscaping( pEvent ) ? apex.util.escapeHTML( pEvent.title ) : pEvent.title ) +
                    '</span><br>';
                tooltip = tooltip + dateDisplay + "<br><br>";

                if ( pEvent.description ) {
                    tooltip = tooltip +
                              ( eventEscaping( pEvent ) ? apex.util.escapeHTML( pEvent.description ) : pEvent.description );
                }
                $( element ).tooltip( {
                                          show        : apex.tooltipManager.defaultShowOption(),
                                          items       : "a.fc-event",
                                          position    : { my: "left top+15", at: "left center", collision: "flipfit" },
                                          content     : tooltip,
                                          tooltipClass: "ui-widget-content ui-corner-all ui-widget ui-tooltip"
                                      } );
            }
        }


        function pWidth( pDivId ) {
            return apex.jQuery( pDivId ).parent().width();
        }

        function tDivs( pContainer, pDivId ) {
            return apex.jQuery( pContainer + " div[id=\"" + pDivId + "\"]" ).length;
        }

        function lPos( pContainer, pDivId ) {
            return apex.jQuery( pContainer + " div[id=\"" + pDivId + "\"]:first" ).parent().position().left;
        }

        function positionObjects( pContainer, pDivId, pInitPos, pWidth ) {
            var lPos = pInitPos;
            apex.jQuery( pContainer + "div[id=\"" + pDivId + "\"]" ).each( function ( index ) {
                apex.jQuery( this ).width( pWidth );
                apex.jQuery( this ).css( 'left', lPos );
                lPos = lPos + pWidth;
            } );
        }

        function setHeight() {
            var lScroller   = apex.jQuery( 'div[id="cal_scroll_div"]' ),
                lViewHeight = Math.round( apex.jQuery( 'div[id="cal_main"]' ).width() / 1.35 );
            var headHeight, allDayHeight, bodyHeight, lHeight, lBody;

            lBody = apex.jQuery( 'table[id="cal_main_table"]' ).find( 'tbody' );
            if ( apex.jQuery( lBody ).length == 1 ) {
                headHeight   = apex.jQuery( lBody ).position().top;
                allDayHeight = apex.jQuery( lScroller ).position().top;
                bodyHeight   = Math.min( // when no scrollbars. +1 for bottom border
                                         lViewHeight - headHeight,
                                         apex.jQuery( 'div[id="cal_days"]' ).height() + allDayHeight + 1
                );
                lHeight      = ( ( bodyHeight - allDayHeight - 1 ) < 500 ? 500 : ( bodyHeight - allDayHeight - 1) );

                lScroller.height( lHeight );
                apex.jQuery( 'div[id="cal_div_filler"]' ).height( lHeight + 50 );
            }
        }

        function setWidth() {
            var lDwidth    = apex.jQuery( "td.m-Calendar-Day.priority1:first" ).width(),
                lHourwidth = apex.jQuery( "th.m-Calendar-HourTitle:first" ).width();
            var lScrollDiv, lSWidth, lIWidth;

            lScrollDiv = apex.jQuery( 'div[id="cal_scroll_div"]' );

            if ( apex.jQuery( lScrollDiv ).length == 1 ) {
                lSWidth = apex.jQuery( lScrollDiv ).get( 0 ).clientWidth;
                lIWidth = apex.jQuery( lScrollDiv ).width();

                if ( !( lIWidth - lSWidth ) ) {
                    apex.jQuery( "td.m-Calendar-DivFiller" ).hide();
                    apex.jQuery( "th.m-Calendar-DivFiller" ).hide();
                } else {
                    apex.jQuery( "td.m-Calendar-DivFiller" ).show();
                    apex.jQuery( "th.m-Calendar-DivFiller" ).show();
                }
                apex.jQuery( "th.m-Calendar-DayOfWeek" ).each( function ( index ) {
                    apex.jQuery( this ).width( lDwidth );
                } );
                apex.jQuery( "td.m-Calendar-AllDay" ).each( function ( index ) {
                    apex.jQuery( this ).width( lDwidth );
                } );

                apex.jQuery( "th.m-Calendar-AlldayTitle" ).each( function ( index ) {
                    apex.jQuery( this ).width( lHourwidth + 1 );
                } );

                //setting the filler width
                apex.jQuery( "th.m-Calendar-HeadFiller" ).width( lHourwidth + Math.round( lHourwidth / 3 ) );
                apex.jQuery( "th.m-Calendar-DayOfWeek:first" ).css( "left", apex.jQuery( "td.m-Calendar-Day.priority1:first" ).position().left
                );
            }
        }

        // *****************************************************************************
        // K E Y B O A R D - S U P P O R T
        // *****************************************************************************

        function initKeyboardMode( view ) {
            var cursorStart;
            var cursorEnd;

            if ( view.name != keyBoardInfo.LIST_VIEW ) {
                if ( view.name == keyBoardInfo.MONTH_VIEW ) {
                    cursorStart = view.start.clone().add( 15, "days" ).startOf( "month" );
                    cursorEnd   = view.start.clone().add( 15, "days" ).startOf( "month" ).add( 1, "days" );
                } else {
                    cursorStart = view.start.clone().hour( gOptions.startingHour );
                    cursorEnd   = view.start.clone().hour( gOptions.startingHour ).add( 60, "minutes" );
                }


                keyBoardInfo.selectorEvent = {
                    id              : keyBoardInfo.SELECTORID,
                    title           : "",
                    allDay          : ( view.name == keyBoardInfo.MONTH_VIEW ),
                    editable        : false,
                    durationEditable: false,
                    className       : keyBoardInfo.cssClass,
                    start           : cursorStart,
                    end             : cursorEnd,
                    overlap         : true
                };

                // prevent mouse actions while keyboard mode is active
                view.options.selectable = false;
                keyBoardInfo.keyboardActive = true;
                if ( !startEventSelection( view ) ) {
                    leaveEventSelection();
                }
            }
        }

        function unrenderSelectorEvent() {
            var view     = gCalendar$.fullCalendar( "getView" ),
                selector = gCalendar$.fullCalendar( "clientEvents", keyBoardInfo.SELECTORID );

            if ( view.name == "month" ) {
                view.unrenderSelection();
            } else {
                if ( selector.length > 0 ) {
                    try {
                        gCalendar$.fullCalendar( "removeEvents", keyBoardInfo.SELECTORID );
                    } catch ( e ) { }
                }
            }
        }

        function renderSelectorEvent( pUpdateOnly ) {
            var view = gCalendar$.fullCalendar( "getView" );
            var calEvent;

            if ( keyBoardInfo.selectorEvent ) {
           
                if ( view.name == "month" ) {
                    view.unrenderSelection();
                    view.renderSelection( keyBoardInfo.selectorEvent );
                } else {
                    // get the event object from FullCalendar ...
                    calEvent = gCalendar$.fullCalendar( 'clientEvents', keyBoardInfo.SELECTORID )[ 0 ];
                    if ( pUpdateOnly && calEvent ) {
                        // adjust start and end timestamps
                        calEvent.start = FC.moment( keyBoardInfo.selectorEvent.start.format( "YYYY-MM-DD[T]HH:mm:ss" ) );
                        calEvent.end   = FC.moment( keyBoardInfo.selectorEvent.end.format( "YYYY-MM-DD[T]HH:mm:ss" ) );
                        // pass event object back to FullCalendar
                        gCalendar$.fullCalendar( "updateEvent", calEvent );
                    } else {
                        try {
                            gCalendar$.fullCalendar( "removeEvents", keyBoardInfo.SELECTORID );
                        } catch ( e ) {
                        }
    
                        // create the event object to be passed to FullCalendar
                        calEvent = {
                            id              : keyBoardInfo.selectorEvent.id,
                            title           : keyBoardInfo.selectorEvent.title,
                            allDay          : keyBoardInfo.selectorEvent.allDay,
                            editable        : keyBoardInfo.selectorEvent.editable,
                            durationEditable: keyBoardInfo.selectorEvent.durationEditable,
                            className       : keyBoardInfo.selectorEvent.className,
                            start           : FC.moment( keyBoardInfo.selectorEvent.start.format( "YYYY-MM-DD[T]HH:mm:ss" ) ),
                            end             : FC.moment( keyBoardInfo.selectorEvent.end.format("YYYY-MM-DD[T]HH:mm:ss" ) ),
                            overlap         : keyBoardInfo.selectorEvent.overlap
                        };
                        // pass event object to FullCalendar
                        gCalendar$.fullCalendar( "renderEvent", calEvent );
                    }
                }
            }
        }

        function startEventSelection( view ) {
            var allEvents = gCalendar$.fullCalendar( "clientEvents" ), i;

            keyBoardInfo.dayEvents = [];
            for ( i in allEvents ) {
                if ( !(
                        (  allEvents[ i ].start.isAfter( view.end.clone().subtract( 1, "minute" ) ) ) ||
                        (  allEvents[ i ].end && allEvents[ i ].end.isBefore( view.start ) ) ||
                        ( !allEvents[ i ].end && allEvents[ i ].start.isBefore( view.start ) ) ||
                        (  allEvents[ i ].id == keyBoardInfo.SELECTORID )
                    ) ) {
                    keyBoardInfo.dayEvents.push( allEvents[ i ] );
                }
            }

            keyBoardInfo.dayEvents.sort( function ( a, b ) {
                var dateA, dateB;

                function getString( a ) {
                    var r;

                    if ( a.end ) { 
                        r = formatYYYYMMDDhhmmss( a.start ) + 
                            "" + ( 9999999999 -  ( ( a.end - a.start ) / 60000 ) ) +
                            a.title;
                    } else {
                        r = formatYYYYMMDDhhmmss( a.start ) + 
                            "9999999999" +
                            a.title;
                    }
                    return r;
                }
                dateA = getString( a );
                dateB = getString( b );

                if ( dateA < dateB ) {
                    return -1;
                } else if ( dateA > dateB ) {
                    return 1;
                } else {
                    return 0;
                }
            } );

            if ( keyBoardInfo.dayEvents.length > 0 ) {
                unrenderSelectorEvent();
                keyBoardInfo.browseMode     = "EVENTS";
                keyBoardInfo.activeDayEvent = 0;
                if ( view.name == "month" ) {
                    openMoreEvents( keyBoardInfo.dayEvents[ keyBoardInfo.activeDayEvent ].start.format( "YYYY-MM-DD" ) );
                }

                focusEvent( view, keyBoardInfo.dayEvents[ keyBoardInfo.activeDayEvent ].id );
            }
            return (keyBoardInfo.dayEvents.length > 0);
        }

        function leaveEventSelection() {
            keyBoardInfo.browseMode     = "CALENDAR";
            keyBoardInfo.activeDayEvent = false;
            closeMoreEvents();
            $( ".fc-event", gCalendar$ ).removeClass( "fc-helper" );
            $( "div.fc-view", gCalendar$ ).focus();
            renderSelectorEvent();
        }

        function leaveKeyboardMode() {
            if ( keyBoardInfo.browseMode == "EVENTS" ) {
                leaveEventSelection();
            }
            unrenderSelectorEvent();
            keyBoardInfo.keyboardActive                             = false;
            gCalendar$.fullCalendar( "getView" ).options.selectable = true;
        }

        function openMoreEvents( pDayYYYYMMDD ) {
            var lMoreLink$ = getMoreLink( pDayYYYYMMDD );

            if ( lMoreLink$ ) {
                lMoreLink$.click();
            }
        }

        function closeMoreEvents( ) {
            $( gCalendar$.find( "span.fc-close.ui-icon.ui-icon-closethick" ) ).click();
        }

        function getDayMinTime( pMoment ) {
            var dayMinTime;
            if ( gOptions.minTime ) {
                dayMinTime = moment( pMoment.format( "YYYY-MM-DD" ) + "T" + gOptions.minTime );
            } else {
                dayMinTime = moment( pMoment.format( "YYYY-MM-DD" ) + "T00:00:00" );
            }
            return dayMinTime;
        }

        function getDayMaxTime( pMoment ) {
            var dayMaxTime;
            if ( gOptions.maxTime ) {
                dayMaxTime = moment( pMoment.format( "YYYY-MM-DD" ) + "T" + gOptions.maxTime );
            } else {
                dayMaxTime = moment( pMoment.format( "YYYY-MM-DD" ) + "T00:00:00" ).add( 1, "days" );
            }
            return dayMaxTime;
        }

        function scrollToHour( hour ) {
            var hourOffset;
            if ( gOptions.minTime ) {
                hourOffset = parseInt( gOptions.minTime.substr( 0, 2 ) );
                hourOffset = (hourOffset < 0 ? 0 : hourOffset);
            } else {
                hourOffset = 0;
            }
            var weekGrid = gCalendar$.find( ".fc-time-grid-container" );
            var hourRow  = gCalendar$.find( ".fc-time-grid-container .fc-axis:odd:eq(" + (hour - hourOffset) + ")" );

            if ( hourRow.length ) {
                var hourRowOffset  = hourRow.position().top;
                var hourRowHeight  = hourRow.height() * 2;
                var weekGridOffset = weekGrid.scrollTop();
                var weekGridHeight = weekGrid.height();
                var scrollDiff     = (hourRowOffset + hourRowHeight) - (weekGridOffset + weekGridHeight);

                if ( hourRowOffset < weekGridOffset ) {
                    weekGrid.animate( { scrollTop: hourRowOffset }, 10 );
                }
                if ( hourRowOffset + hourRowHeight > weekGridOffset + weekGridHeight ) {
                    weekGrid.animate( { scrollTop: weekGridOffset + Math.max( hourRowHeight, scrollDiff ) }, 10 );
                }
            }
        }

        //
        // This function moved the event selection forward or backwards.
        //
        function gotoEvent( pView, pDirection ) {
            $( ".fc-event", gCalendar$ ).removeClass( "fc-helper" );
 
            if ( pDirection == -1 ) {
                if ( keyBoardInfo.activeDayEvent > 0 ) {
                    keyBoardInfo.activeDayEvent--;
                } else {
                    keyBoardInfo.activeDayEvent = keyBoardInfo.dayEvents.length - 1;
                }
            } else {
                if ( keyBoardInfo.activeDayEvent < keyBoardInfo.dayEvents.length - 1 ) {
                    keyBoardInfo.activeDayEvent++;
                } else {
                    keyBoardInfo.activeDayEvent = 0;
                }
            }
            if ( pView.name == keyBoardInfo.MONTH_VIEW ) {
                closeMoreEvents( );
            }

            if ( pView.name == keyBoardInfo.MONTH_VIEW ) {
                if ( keyBoardInfo.dayEvents[ keyBoardInfo.activeDayEvent ].start.isBefore( pView.start ) ) {
                    openMoreEvents( pView.start.format( "YYYY-MM-DD") );
                } else {
                    openMoreEvents( keyBoardInfo.dayEvents[ keyBoardInfo.activeDayEvent ].start.format( "YYYY-MM-DD" ) );
                }
            }

            focusEvent( pView, keyBoardInfo.dayEvents[ keyBoardInfo.activeDayEvent ].id );
        }

        //
        // This function sets the focus to a specific ID
        //
        function focusEvent( pView, pEventId ) {
            var focusEvent$;

            focusEvent$ = $( ".fc-event[data-id='" + 
                             util.escapeHTML( pEventId ) + 
                             "']", gCalendar$ );

            focusEvent$.addClass( "fc-helper" );
            if ( focusEvent$ ) { 
                if ( pView.name == keyBoardInfo.MONTH_VIEW ) {
                    if ( focusEvent$.length === 1 ) {
                        focusEvent$.focus(); 
                    } else {
                        focusEvent$.each( function ( i, o ) { if ( $( o ).closest( ".fc-popover" ) ) { $( o ).focus(); } } );
                    }
                } else {
                    focusEvent$.first().focus();
                }
            }
        }

        //
        // This function moves the new event selection forward or backwards
        //
        function moveSelection( pView, pDirection, pAmount, pUnit ) {
            var changeView = false;
            var isAllowed  = false;

            switch ( pView.name ) {
                case keyBoardInfo.MONTH_VIEW:
                    isAllowed = true;
                    if ( pUnit == "days" ) {
                        var startOfMonth = keyBoardInfo.selectorEvent.start.clone().startOf( "month" );
                        var endOfMonth   = keyBoardInfo.selectorEvent.start.clone().endOf( "month" );
                        while ( !gOptions.weekEnds && startOfMonth.format( "E" ) > 5 ) {
                            startOfMonth.add( 1, pUnit );
                        }
                        while ( !gOptions.weekEnds && endOfMonth.format( "E" ) > 5 ) {
                            endOfMonth.subtract( 1, pUnit );
                        }
                        if (
                            ( pDirection == 1 &&
                              ( !keyBoardInfo.selectorEvent.start.clone().add( pAmount, pUnit ).isBefore( endOfMonth ) )
                            ) ||
                            ( pDirection == -1 &&
                              ( keyBoardInfo.selectorEvent.start.clone().subtract( pAmount, pUnit ).isBefore( startOfMonth ) )
                            )
                        ) {
                            changeView = true;
                        }
                    }
                    break;
                case keyBoardInfo.WEEK_VIEW:
                    if ( pUnit == "days" ) {
                        isAllowed = true;
                        if (
                            ( pDirection == 1 &&
                              ( !keyBoardInfo.selectorEvent.start.clone().add( pAmount, pUnit ).isBefore( pView.end ) )
                            ) ||
                            ( pDirection == -1 &&
                              ( keyBoardInfo.selectorEvent.start.clone().subtract( pAmount, pUnit ).isBefore( pView.start ) )
                            )
                        ) {
                            changeView = true;
                        }
                    } else { // minutes
                        if (
                            ( pDirection == 1 &&
                              ( !(keyBoardInfo.selectorEvent.end.clone().add( pAmount, pUnit ).isAfter( getDayMaxTime( keyBoardInfo.selectorEvent.end.clone().subtract( 1, "minute" ) ) )) )
                            ) ||
                            ( pDirection == -1 &&
                              ( !(keyBoardInfo.selectorEvent.start.clone().subtract( pAmount, pUnit ).isBefore( getDayMinTime( keyBoardInfo.selectorEvent.start ) )) )
                            )
                        ) {
                            isAllowed = true;
                        }
                    }
                    break;
                case keyBoardInfo.DAY_VIEW:
                    if ( pUnit == "days" ) {
                        changeView = true;
                        isAllowed  = true;
                    } else {
                        if (
                            ( pDirection == 1 &&
                              ( !(keyBoardInfo.selectorEvent.end.clone().add( pAmount, pUnit ).isAfter( getDayMaxTime( pView.start ) )) )
                            ) ||
                            ( pDirection == -1 &&
                              ( !(keyBoardInfo.selectorEvent.start.clone().subtract( pAmount, pUnit ).isBefore( getDayMinTime( pView.start ) )) )
                            )
                        ) {
                            isAllowed = true;
                        }
                    }
                    break;
            }

            if ( isAllowed ) {
                keyBoardInfo.selectorEvent.start.add( pAmount * pDirection, pUnit );
                keyBoardInfo.selectorEvent.end.add( pAmount * pDirection, pUnit );

                if ( pUnit == "days" ) {
                    while ( !gOptions.weekEnds && keyBoardInfo.selectorEvent.start.format( "E" ) > 5 ) {
                        keyBoardInfo.selectorEvent.start.add( pDirection, pUnit );
                        keyBoardInfo.selectorEvent.end.add( pDirection, pUnit );
                    }
                }

                if ( changeView ) {
                    keyBoardInfo.viewChangeByKey = true;
                    gCalendar$.fullCalendar( "gotoDate", keyBoardInfo.selectorEvent.start );
                    renderSelectorEvent( false );

                    if ( pView.name == keyBoardInfo.WEEK_VIEW || pView.name == keyBoardInfo.DAY_VIEW ) {
                        scrollToHour( keyBoardInfo.selectorEvent.start.hour() );
                    }
                } else {
                    renderSelectorEvent( true );
                }

                if ( pView.name == keyBoardInfo.WEEK_VIEW || pView.name == keyBoardInfo.DAY_VIEW ) {
                    scrollToHour( keyBoardInfo.selectorEvent.start.hour() );
                }

            }
        }

        function extendSelection( pView, pDirection, pAmount, pUnit ) {
            var isAllowed = false;

            if ( pDirection == -1 ) {
                if ( keyBoardInfo.selectorEvent.end.clone().subtract( pAmount, pUnit ).isAfter( keyBoardInfo.selectorEvent.start ) ) {
                    isAllowed = true;
                }
            } else {
                switch ( pView.name ) {
                    case keyBoardInfo.MONTH_VIEW:
                        if ( pUnit == "days" ) {
                            if ( keyBoardInfo.selectorEvent.end.clone().add( pAmount, pUnit ).subtract( 1, "minutes" ).isBefore( pView.end ) ) {
                                isAllowed = true;
                            }
                        }
                        break;
                    case keyBoardInfo.WEEK_VIEW:
                        if ( pUnit == "days" ) {
                            if ( keyBoardInfo.selectorEvent.end.clone().add( pAmount, pUnit ).subtract( 1, "minute" ).isBefore( pView.end ) ) {
                                isAllowed = true;
                            }
                        } else {
                            if ( !(keyBoardInfo.selectorEvent.end.clone().add( pAmount, pUnit ).isAfter( getDayMaxTime( keyBoardInfo.selectorEvent.end.clone().subtract( 1, "minute" ) ) )) ) {
                                isAllowed = true;
                            }
                        }
                        break;
                    case keyBoardInfo.DAY_VIEW:
                        if ( pUnit == "minutes" ) {
                            if ( !(keyBoardInfo.selectorEvent.end.clone().add( 30, "minutes" ).isAfter( getDayMaxTime( pView.start ) )) ) {
                                isAllowed = true;
                            }
                        }
                        break;
                }
            }
            if ( isAllowed ) {
                keyBoardInfo.selectorEvent.end.add( pAmount * pDirection, pUnit );
                if ( pUnit == "days" ) {
                    while ( !gOptions.weekEnds && keyBoardInfo.selectorEvent.end.clone().subtract( 1, "minute" ).format( "E" ) > 5 ) {
                        keyBoardInfo.selectorEvent.end.add( pDirection, "days" );
                    }
                }
                renderSelectorEvent( true );
            }
            if ( pView.name == keyBoardInfo.DAY_VIEW || pView.name == keyBoardInfo.WEEK_VIEW ) {
                scrollToHour( keyBoardInfo.selectorEvent.end.clone().subtract( 1, "minutes" ).hour() );
            }
        }


        function processKeyEvents( view, e ) {
            switch ( e.which ) {
                case 27:
                    if ( keyBoardInfo.browseMode == "CALENDAR" ) {
                        if ( !startEventSelection( view ) ) {
                            leaveKeyboardMode();
                        }
                    } else {
                        leaveKeyboardMode();
                    }
                    e.preventDefault();
                    break;

                case 78:
                    if ( e.shiftKey && keyBoardInfo.browseMode == "EVENTS" ) {
                        leaveEventSelection();
                    }
                    e.preventDefault();
                    break;

                case 13:
                case 32:
                    if ( keyBoardInfo.browseMode == "CALENDAR" ) {
                        _calSelect( keyBoardInfo.selectorEvent.start.clone(), keyBoardInfo.selectorEvent.end.clone(), gAllDay );
                    } else {
                        _eventClick( keyBoardInfo.dayEvents[ keyBoardInfo.activeDayEvent ] );
                    }
                    e.preventDefault();
                    break;

                case 38:
                    if ( keyBoardInfo.browseMode == "EVENTS" ) {
                        gotoEvent( view, -1 );
                    } else {
                        if ( view.name == keyBoardInfo.WEEK_VIEW || view.name == keyBoardInfo.DAY_VIEW ) {
                            if ( e.shiftKey ) {
                                extendSelection( view, -1, 30, "minutes" );
                            } else {
                                moveSelection( view, -1, 30, "minutes" );
                            }
                        } else {
                            if ( e.shiftKey ) {
                                extendSelection( view, -1, 7, "days" );
                            } else {
                                moveSelection( view, -1, 7, "days" );
                            }
                        }
                    }
                    e.preventDefault();
                    break;
                case 9:
                    leaveKeyboardMode();
                    break;
                case 40:
                    if ( keyBoardInfo.browseMode == "EVENTS" ) {
                        gotoEvent( view, 1 );
                    } else {
                        if ( view.name == keyBoardInfo.WEEK_VIEW || view.name == keyBoardInfo.DAY_VIEW ) {
                            if ( e.shiftKey ) {
                                extendSelection( view, 1, 30, "minutes" );
                            } else {
                                moveSelection( view, 1, 30, "minutes" );
                            }
                        } else {
                            if ( e.shiftKey ) {
                                extendSelection( view, 1, 7, "days" );
                            } else {
                                moveSelection( view, 1, 7, "days" );
                            }
                        }
                    }
                    e.preventDefault();
                    break;
                case keyBoardInfo.KEY_NEXT:
                    if ( keyBoardInfo.browseMode == "CALENDAR" ) {
                        if ( !e.shiftKey ) {
                            moveSelection( view, 1, 1, "days" );
                        } else {
                            extendSelection( view, 1, 1, "days" );
                        }
                    }
                    e.preventDefault();
                    break;
                case keyBoardInfo.KEY_PREV:
                    if ( keyBoardInfo.browseMode == "CALENDAR" ) {
                        if ( !e.shiftKey ) {
                            moveSelection( view, -1, 1, "days" );
                        } else {
                            extendSelection( view, -1, 1, "days" );
                        }
                    }
                    e.preventDefault();
                    break;
                case 33:  // PGUP
                    if ( keyBoardInfo.browseMode == "CALENDAR" ) {
                        switch ( view.name ) {
                            case keyBoardInfo.WEEK_VIEW:
                                moveSelection( view, -1, 7, "days" );
                                break;
                            case keyBoardInfo.DAY_VIEW:
                                moveSelection( view, -1, 1, "days" );
                                break;
                            case keyBoardInfo.MONTH_VIEW:
                                moveSelection( view, -1, view.start.clone().subtract( 15, "days" ).endOf( "month" ).date(), "days" );
                                break;
                        }
                    }
                    e.preventDefault();
                    break;
                case 34:  // PGDOWN
                    if ( keyBoardInfo.browseMode == "CALENDAR" ) {
                        switch ( view.name ) {
                            case keyBoardInfo.WEEK_VIEW:
                                moveSelection( view, 1, 7, "days" );
                                break;
                            case keyBoardInfo.DAY_VIEW:
                                moveSelection( view, 1, 1, "days" );
                                break;
                            case keyBoardInfo.MONTH_VIEW:
                                moveSelection( view, 1, view.start.clone().add( 15, "days" ).endOf( "month" ).date(), "days" );
                                break;
                        }
                    }
                    e.preventDefault();
                    break;

            }
        }

        // *****************************************************************************
        // E X P O S E D   F U N C T I O N S
        // *****************************************************************************

        widget.cssCalendar.download = function ( pFormat ) {

            var lView, lUrl, lWindows;

            lView = gCalendar$.fullCalendar( 'getView' ),
            lUrl  = server.pluginUrl( 
                gOptions.ajaxIdentifier, 
                {
                    x01      : "DOWNLOAD",
                    x02      : formatYYYYMMDDhhmmss( lView.start ),
                    x03      : formatYYYYMMDDhhmmss( lView.end ),
                    x06      : lView.name,
                    x07      : lView.title,
                    x10      : pFormat,
                    pageItems: gOptions.pageItems 
                } 
            );

            lWindows = window.open( lUrl );
            lWindows.focus();
        };

        /* function to show the day specific calendar event on Tap */
        widget.cssCalendar.mobileDayTap = function ( pRegionId, pThis ) {
            var lDate     = apex.jQuery( pThis ).data( "date" );
            var lDetails$ = apex.jQuery( "#calendar_day_details_" + pRegionId, apex.gPageContext$ );
            apex.jQuery( ".m-Calendar-day" ).removeClass( "is-active" );
            apex.jQuery( pThis ).addClass( "is-active" );
            apex.widget.cssCalendar.getDayData( pRegionId, lDate, {
                clear  : function () {
                    lDetails$.empty();
                },
                success: function ( pData ) {
                    lDetails$.html( pData );
                    apex.event.trigger(
                        gRegion$,
                        'apexcalendardateselect',
                        { "newStartDate": lDate, "newEndDate": lDate }
                    );
                }
            } );
        };

        /* function to load the current date's data after the calendar loads */
        widget.cssCalendar.mobileMonthLoad = function ( pRegionId, pDate ) {
            var lDetails$    = apex.jQuery( "#calendar_day_details_" + pRegionId, apex.gPageContext$ );
            gOptions.curDate = pDate;
            apex.widget.cssCalendar.getDayData( pRegionId, pDate, {
                clear  : function () { lDetails$.empty();},
                success: function ( pData ) {lDetails$.html( pData );}
            } );
        };

        widget.cssCalendar.executeMobileCreateLink = function ( pRegionId, pStart, pEnd ) {
            if ( gOptions.createLink && gOptions.createLink !== "" ) {
                // For the CREATE LINK we add the selected START and END dates in JS code
                // if page checksum is enabled we need the server to recalculate the URL checksum
                server.plugin( gOptions.ajaxIdentifier, {
                    x01: "PREPARE_URL", /* action */
                    x02: pStart,
                    x03: pEnd,
                    x06: gOptions.createLink
                }, {
                    success: function ( pData ) {
                        if ( pData.url ) {
                            window.location.href = pData.url;
                        }
                    }
                } );
            } else {
                if ( pRegionId ) {
                    // when no edit link is defined, fire event
                    apex.event.trigger(
                        $( "#" + pRegionId ),
                        'apexcalendardateselect',
                        { "newStartDate": pStart, "newEndDate": pEnd }
                    );
                }
            }
        };


        /* function to get the data of the date which is tapped or clicked */
        widget.cssCalendar.getDayData = function ( pRegionId, pDate ) {
            gOptions.curDate = pDate;
            server.plugin( gOptions.ajaxIdentifier,
                           {
                               x01      : "GETDATA", /* action */
                               x09      : pDate,
                               x10      : pRegionId,
                               pageItems: gOptions.pageItems
                           }, {
                               dataType     : "html",
                               refreshObject: gRegion$,
                               success      : function ( pData ) {
                                   apex.jQuery( '#calendar_day_details_' + pRegionId, apex.gPageContext$ ).html( pData ).trigger( "create" );
                               }
                           } );

        }; //getDayData

        widget.cssCalendar.refreshMobileCalendar = function () {
            server.plugin( gOptions.ajaxIdentifier, {
                x01      : gOptions.mobileCurType,
                x02      : gOptions.mobileCurStartDate,
                x03      : gOptions.mobileCurEndDate,
                x09      : gOptions.mobileCurStartDate,
                pageItems: gOptions.pageItems
            }, {
                dataType     : "html",
                refreshObject: gRegion$,
                success      : function ( pData ) {
                    apex.jQuery( gRegion$, apex.gPageContext$ ).html( pData ).trigger( "create" );
                }
            } );
        };

        /* main function to display the mobile calendar */
        widget.cssCalendar.getMobileCalendar = function ( pTypeAction, pStart, pEnd, pCurrent, pCallback ) {
            gOptions.startDate = pStart;
            gOptions.endDate   = pEnd;
            gOptions.curDate   = pCurrent;

            server.plugin( gOptions.ajaxIdentifier, {
                x01      : pTypeAction,
                x02      : gOptions.startDate,
                x03      : gOptions.endDate,
                x09      : gOptions.curDate,
                pageItems: gOptions.pageItems
            }, {
                dataType     : "html",
                refreshObject: gRegion$,
                success      : function ( pData ) {
                    apex.jQuery( gRegion$, apex.gPageContext$ ).html( pData ).trigger( "create" );
                }
            } );
        };

        widget.cssCalendar.fireViewChangeEvent = function ( pRegionId, pType, pStart, pEnd ) {
            gOptions.mobileCurStartDate = pStart;
            gOptions.mobileCurEndDate   = pEnd;
            gOptions.mobileCurType      = pType;

            apex.event.trigger(
                $( "#" + pRegionId ),
                "apexcalendarviewchange",
                {
                    "viewType" : pType,
                    "startDate": pStart ,
                    "endDate"  : pEnd
                }
            );
        };


        widget.cssCalendar.arrangeObjects = function ( pContainer, pCount ) {
            var lWidth, lParentwidth, i;
            setHeight();
            setWidth();
            for ( i = 1; i <= pCount; i++ ) {
                lParentwidth = pWidth( pContainer + 'div[id=day' + i + ']' );
                lWidth       = Math.round( (lParentwidth - Math.round( lParentwidth / 20 )) / tDivs( pContainer, 'day' + i ) );
                positionObjects( pContainer, 'day' + i, lPos( pContainer, 'day' + i ), lWidth );
            }
        };

        widget.cssCalendar.orientationchange = function ( pCaltype, pEvent ) {
            if ( pEvent.orientation === 'landscape' ) {
                if ( apex.jQuery( window ).width() <= 480 ) { //devices with width similar to iPhone
                    if ( pCaltype === 'WEK' ) {
                        apex.widget.cssCalendar.getMobileCalendar( 'WKS_SAME', gOptions.startDate, gOptions.endDate, gOptions.curDate, '' );
                    }
                }
            } else {
                if ( apex.jQuery( window ).width() <= 480 && ( pCaltype == 'WEK' || pCaltype == 'WKS') ) {
                    apex.widget.cssCalendar.getMobileCalendar( 'LST_SAME', gOptions.startDate, gOptions.endDate, gOptions.curDate, '' );
                }
            }
        };

    }; // cssCalendar

})( apex.widget, apex.util, apex.server, apex.jQuery );
